@@ -17,16 +17,16 @@
      (make-param-proc-class-object
       "instance of :param-proc"
       (list normal-tvar jump-tvar)
-      (translate-simple-proc-class-expression
-       gl-empty-binder
-       (list
-	(translate-general-proc-type-expression
-	 gl-empty-binder #t
-	 (list
-	  (translate-general-proc-type-expression
-	   gl-empty-binder #t (list jump-tvar) tt-none #t #f #t #f))
+      (make-tpti-general-proc
+       #t
+       (make-tuple-type
+	(make-tpti-general-proc
+	 #t
+	 (make-tuple-type
+	  (make-tpti-general-proc
+	   #t (make-tuple-type jump-tvar) tt-none #t #f #t #f))
 	 normal-tvar #t #f #f #f))
-       (get-union-of-types gl-empty-binder (list normal-tvar jump-tvar))
+       (make-tt-union normal-tvar jump-tvar)
        #t #f #f #f)))))
 
 
@@ -38,30 +38,30 @@
      (make-param-proc-class-object
       "instance of :param-proc"
       (list normal-tvar jump-tvar)
-      (translate-simple-proc-class-expression
-       gl-empty-binder
-       (list
-	(translate-general-proc-type-expression
-	 gl-empty-binder #t
-	 (list
-	  (translate-general-proc-type-expression
-	   gl-empty-binder #t (list jump-tvar) tt-none #t #f #t #f))
+      (make-tpti-general-proc
+       #t
+       (make-tuple-type
+	(make-tpti-general-proc
+	 #t
+	 (make-tuple-type
+	  (make-tpti-general-proc
+	   #t (make-tuple-type jump-tvar) tt-none #t #f #t #f))
 	 normal-tvar #f #f #f #f))
-       (get-union-of-types gl-empty-binder (list normal-tvar jump-tvar))
+       (make-tt-union normal-tvar jump-tvar)
        #f #f #f #f)))))
 
 
 (define expr-call/cc-without-result
   (get-prim-proc-expression
    '_b_call/cc-without-result
-   (translate-simple-proc-class-expression
-    gl-empty-binder
-    (list
-     (translate-general-proc-type-expression
-      gl-empty-binder #t
-      (list
-       (translate-general-proc-type-expression
-	gl-empty-binder #t '() tt-none #t #f #t #f))
+   (make-tpti-general-proc
+    #t
+    (make-tuple-type
+     (make-tpti-general-proc
+      #t
+      (make-tuple-type
+       (make-tpti-general-proc
+	#t (make-tuple-type) tt-none #t #f #t #f))
       tt-none #f #f #f #f))
     tt-none
     #f #f #f #f)))
