@@ -430,7 +430,7 @@
 	    (raise 'trying-to-set-nonmutable-variable))
 	   ((hfield-ref variable 'read-only?)
 	    (raise 'attempted-setting-of-read-only-variable))
-	   ((entity-type-is-none? value-expr)
+	   ((entity-type-is-none1? (compiler-get-binder compiler) value-expr)
 	    (raise 'set-expression-with-type-none))
 	   ;; We need not check may-change-variable? for module
 	   ;; references since they are toplevel variables.
@@ -501,23 +501,24 @@
 			(translate-expr compiler env #f 
 					(list-ref s-field 
 						  i-source-field-init-value))
-			'())))
+			'()))
+		   (binder (compiler-get-binder compiler)))
 	      (cond
-	       ((entity-is-none? r-field-type)
+	       ((entity-is-none1? binder r-field-type)
 		(raise 'none-as-a-field-type))
 	       ((not (memv r-read-access '(hidden module public)))
 		(raise 'invalid-read-access))
 	       ((not (memv r-write-access '(hidden module public)))
 		(raise 'invalid-write-access))
 	       ((not (is-t-subtype?
-		      (compiler-get-binder compiler)
+		      binder
 		      (get-entity-type r-field-type)
 		      tt-type))
 		(raise 'invalid-field-type))
 	       ((and has-init-value?
 		     (not
 		      (is-t-instance?
-		       (compiler-get-binder compiler)
+		       binder
 		       r-init-value
 		       r-field-type)))
 		(raise (list 'field-initializer-type-mismatch
@@ -1169,13 +1170,14 @@
 			     env1
 			     #f 
 			     (caddr varspec)))
+		 (binder (compiler-get-binder compiler))
 		 (result
 		  (cond
 		   ((and (not (hfield-ref compiler 'inside-param-def?))
-			 (entity-is-none? type))
+			 (entity-is-none1? binder type))
 		    (raise 'none-as-declared-type))
 		   ((and (not (hfield-ref compiler 'inside-param-def?))
-			 (entity-type-is-none? init-expr))
+			 (entity-type-is-none1? binder init-expr))
 		    (raise (list 'initializer-expression-with-type-none
 				 (cons 's-name name))))
 		   (else
@@ -1188,7 +1190,7 @@
 			   ((null? to-init-type)
 			    (raise 'internal-invalid-letvar-type-2))
 			   ((is-t-subtype?
-			     (compiler-get-binder compiler)
+			     binder
 			     to-init-type
 			     type)
 			    (let ((variable
@@ -1213,8 +1215,7 @@
 				(compiler-alloc-loc compiler name #f)
 				type
 				#t
-				(is-final-class?
-				 (compiler-get-binder compiler) type)
+				(is-final-class? binder type)
 				#f
 				volatile?
 				#f
@@ -1369,6 +1370,7 @@
 
 (define (translate-let-expression compiler env1 expr-tail constant-bindings?
 				  volatile?)
+  ;; TBD: Raise an exception if both constant-bindings? and volatile? are #t.
   (dwl2 "t-let-e")
   (assert (hrecord-is-instance? compiler <compiler>))
   (assert (hrecord-is-instance? env1 <environment>))
@@ -1425,6 +1427,8 @@
 			'()
 			let-expr-pure?
 			#f
+			;; TBD: Should we set need-revision? to #f if
+			;; type-dispatched? = #t ?
 			(hfield-ref compiler 'inside-param-def?)
 			subexpr-value
 			always-returns?        ; <dynamic-expression>
@@ -1438,6 +1442,7 @@
 
 (define (translate-letrec-expression compiler env expr-tail
 				     constant-bindings? volatile? order?)
+  ;; TBD: Raise an exception if both constant-bindings? and volatile? are #t.
   (dwl4 "translate-letrec-expression")
   (if (< (length expr-tail) 2)
       (raise 'too-short-letrec)
@@ -1457,8 +1462,11 @@
 	     (exact-type? (hfield-ref t-body 'exact-type?))
 	     (subexpr-pure? (is-pure-entity? t-body))
 	     (subexpr-value (get-entity-value t-body))
-	     (init-exprs (map pick5th letrec-vars))
 	     (type-dispatched? (entity-type-dispatched? t-body))
+	     (init-exprs (map pick5th letrec-vars))
+	     (init-exprs-pure?
+	      (and-map? is-pure-entity? init-exprs))
+	     (letrec-pure? (and subexpr-pure? init-exprs-pure?))
 	     (always-returns?
 	      (and (entity-always-returns? t-body)
 		   (and-map? entity-always-returns? init-exprs)))
@@ -1475,7 +1483,7 @@
 		      type-dispatched?
 		      exact-type?
 		      '()
-		      subexpr-pure?
+		      letrec-pure?
 		      #f
 		      (hfield-ref compiler 'inside-param-def?)
 		      subexpr-value
@@ -1499,6 +1507,7 @@
 	(inside-param-def? (hfield-ref compiler 'inside-param-def?))
 	(binder (compiler-get-binder compiler)))
     (dwl4 "translate-genproc-appl/1")
+    ;; TBD: Should we check that expr-head is a target object?
     (translate-general-genproc-appl
      binder
      expr-head arglist #t)))
@@ -1518,29 +1527,19 @@
 				  (hfield-ref compiler 'inside-param-def?)
 				  expr-head arglist '())))
 
-;; (if (hfield-ref compiler 'inside-param-def?)
-;; 	 (translate-param-proc-appl-in-param-proc compiler symtbl
-;; 						  expr-head arglist)
-;; 	 (translate-normal-param-proc-appl (hfield-ref compiler 'param-cache)
-;; 					   expr-head arglist))))
-
 
 (define (translate-simple-proc-appl compiler symtbl expr-head expr-tail)
   (dwl3 "translate-simple-proc-appl ENTER")
   (let ((arguments (map* (lambda (expr)
 			   (translate-expr compiler symtbl #f expr))
 			 expr-tail)))
-
-    ;; TBR
-    (if (hrecord-is-instance? expr-head <expr-constructor>)
-	(dwl1 "constructor HEP"))
-     
     (dvar1-set! expr-head)
     (let ((result '()))
       (set! result
 	    ;; Procedures field-ref and field-set!
 	    ;; are handled differently by the compiler
 	    ;; and the linker. That's why we handle them here.
+	    ;; TBD: use eq? instead of eqv?
 	    (cond
 	     ((eqv? expr-head tp-field-ref)
 	      (translate-field-ref-appl compiler expr-head arguments))
@@ -1596,6 +1595,7 @@
       (let ((member-type (translate-expr compiler symtbl #f (car expr-tail)))
  	    (binder (compiler-get-binder compiler)))
 	(translate-vector-expression binder (list member-type)))
+      ;; TBD: remove "uniform"
       (raise 'invalid-uniform-vector-expression)))
 
 
@@ -1667,22 +1667,6 @@
 	  (map (lambda (subexpr) (translate-expr compiler symtbl #f subexpr))
 	       expr-tail)))
     (strong-assert (and-map? is-target-object? r-type-params))
-;;    (check-no-none-types r-type-params)
-
-    ;; TBR
-    ;; (let ((address (hfield-ref r-param-ltype 'address)))
-    ;;   (if (and (not-null? address)
-    ;; 	       (eq? (hfield-ref address 'source-name) ':my-type))
-    ;; 	  (begin
-    ;; 	    (dwl2 "my-type HEP")
-    ;; 	    (set! gl-counter13 (+ gl-counter13 1))
-    ;; 	    (dwl2 gl-counter13)
-    ;; 	    (if (= gl-counter13 2)
-    ;; 	    	(begin
-    ;; 	    	  (dvar1-set! r-param-ltype)
-    ;; 	    	  (dvar2-set! r-type-params)
-    ;; 	    	  (raise 'stop-my-type-2))))))
- 
     (let ((result
 	   (if (hfield-ref r-param-ltype 'incomplete?)
 	       (make-apti r-param-ltype r-type-params)
@@ -1690,22 +1674,6 @@
 		(compiler-get-binder compiler)
 		r-param-ltype
 		r-type-params))))
-
-      ;; TBR
-      ;; (let ((address (hfield-ref r-param-ltype 'address)))
-      ;; 	(if (and (not-null? address)
-      ;; 		 (eq? (hfield-ref address 'source-name) ':my-type))
-      ;; 	    (begin
-      ;; 	      (dwl2 "my-type HEP")
-      ;; 	      (set! gl-counter13 (+ gl-counter13 1))
-      ;; 	      (dwl2 gl-counter13)
-      ;; 	      (if (= gl-counter13 2)
-      ;; 		  (begin
-      ;; 		    (dvar1-set! result)
-      ;; 		    (dvar2-set! r-param-ltype)
-      ;; 		    (dvar3-set! r-type-params)
-      ;; 		    (raise 'stop-my-type-2))))))
-
       result)))
 
 
@@ -1757,6 +1725,7 @@
 			     (translate-expr compiler symtbl #f expr))
 			   s-arg-desc-expr))
 		     (result-type
+		      ;; TBD: This check should probably be removed.
 		      (if (not-null? s-result-expr)
 			  (translate-expr compiler symtbl #f
 					  s-result-expr)
@@ -1790,7 +1759,7 @@
 
 
 (define (translate-param-prim-proc compiler symtbl expr-tail check?)
-  (dwl4 "translate-primitive-procedure")
+  (dwl4 "translate-param-prim-proc")
   (assert (hrecord-is-instance? compiler <compiler>))
   (assert (hrecord-is-instance? symtbl <environment>))
   (assert (list? expr-tail))
@@ -1890,8 +1859,8 @@
 	(if (not (symbol? s-jump-proc-name))
 	    (raise 'exec/cc:invalid-proc-name)
 	    (let* ((r-jump-type (translate-expr compiler symtbl #f s-jump-type))
-		   (jump-takes-arg? (not (entity-is-none? r-jump-type)))
 		   (binder (compiler-get-binder compiler))
+		   (jump-takes-arg? (not (entity-is-none1? binder r-jump-type)))
 		   (r-jump-proc-type
 		    (if jump-takes-arg?
 			(translate-simple-proc-class-expression
@@ -1914,8 +1883,8 @@
 		   (pure? (is-pure-entity? r-body))
 		   (tmp3 (begin (dwl4 "translate-exec/cc-3") 0))
 		   (r-result-type
-		    (if (or (entity-is-none? r-body-type)
-			    (entity-is-none? r-jump-type))
+		    (if (or (entity-is-none1? binder r-body-type)
+			    (entity-is-none1? binder r-jump-type))
 			tt-none
 			(get-union-of-types
 			 binder (list r-body-type r-jump-type))))
@@ -1927,12 +1896,11 @@
 		    (make-exec/cc-body-proc
 		     r-body r-body-proc-type r-jump-proc-type jump-var
 		     s-jump-proc-name))
-		   (binder (compiler-get-binder compiler))
 		   (final? (is-final-class?
 			    binder r-result-type))
 		   (proc (make-var-ref-to-var
 			  (cond
-			   ((entity-is-none? r-result-type)
+			   ((entity-is-none1? binder r-result-type)
 			    var-call/cc-without-result)
 			   (pure?
 			    var-call/cc)
@@ -1966,6 +1934,7 @@
 (define (parse-signature-member-sexpr compiler symtbl s-member)
   (if (not (and (list? s-member) (= (length s-member) 4)))
       (begin
+	;; TBD: fix this error
 	(display s-member)
 	(raise 'signature-syntax-error))
       (let ((s-name (car s-member))
