@@ -94,7 +94,7 @@
 	       (else
 		(dvar1-set! expr-head)
 
-		;; TBR
+		;; TBD: fix error handling
 		(display (target-object-as-string type-expr))
 		(newline)
 		
@@ -102,6 +102,8 @@
     (if (not-null? translated-expr)
 	translated-expr
 	(begin
+	  ;; We should never arrive here (?).
+	  ;; TBD: fix the error message
 	  (write-line expr-head)
 	  (write-line expr-tail)
 	  (raise 'could-not-parse-expression)))))
@@ -146,6 +148,7 @@
 
 
 ;; We allow casting an expression to <none>.
+;; TBD: fix handling sexpr-default
 (define (translate-cast-expression compiler symtbl sexpr-type sexpr-to-eval
 				   sexpr-default type-check?)
   (dwl3 "translate-cast-expression")
@@ -154,13 +157,13 @@
 	(t-default-expr
 	 (if (null? sexpr-default)
 	     empty-expression
-	     (translate-expr compiler symtbl #f sexpr-default))))
-    (if (entity-type-is-none? t-value-expr)
+	     (translate-expr compiler symtbl #f sexpr-default)))
+	(binder (compiler-get-binder compiler)))
+    (if (entity-type-is-none1? binder t-value-expr)
 	(raise 'argument-with-type-none-in-cast))
     (let ((tt (if (not (is-t-type-variable? t-type-expr))
 		  t-type-expr
-		  '()))
-	  (binder (compiler-get-binder compiler)))
+		  '())))
       (dvar1-set! t-type-expr)
       (cond
        ((and (not (hfield-ref compiler 'inside-param-def?))
@@ -185,16 +188,21 @@
 	       (result
 		(make-hrecord <cast-expression>
 			      result-type
+			      ;; TBD: check that t-default-expr is dispatched,
+			      ;; too.
 			      (entity-type-dispatched? t-value-expr)
 			      (and (not-null? tt)
 				   (is-final-class? binder tt))
 			      '()
 
+			      ;; TBD: check that t-default-expr is pure, too.
 			      (is-pure-entity? t-value-expr)
 			      #f
 			      (hfield-ref compiler 'inside-param-def?)
 			      '()
 
+			      ;; TBD: check that t-default-expr returns always,
+			      ;; too. No such check for never-returns?.
 			      (entity-always-returns? t-value-expr)
 			      (entity-never-returns? t-value-expr)
 
@@ -273,17 +281,14 @@
   (let ((argc (length expr-tail)))
     (if (or (= argc 2) (= argc 3))
 	(let* ((t-condition (translate-expr compiler symtbl #f (car expr-tail)))
-	       (tmp1 (begin (dwl3 "xlat-kw-if/1") 0))
 	       (t-expr-true
 		(begin
 		  (dwl4 (cadr expr-tail))
 		  (translate-expr compiler symtbl #f (cadr expr-tail))))
-	       (tmp2 (begin (dwl3 "xlat-kw-if/2") 0))
 	       (t-expr-false
 		(if (= argc 3)
 		    (translate-expr compiler symtbl #f (caddr expr-tail))
 		    empty-expression))
-	       (tmp3 (begin (dwl3 "xlat-kw-if/3") 0))
 	       (binder (compiler-get-binder compiler)))
 	  (translate-if-expression binder
 				   t-condition t-expr-true t-expr-false
@@ -350,6 +355,7 @@
 	(raise 'invalid-lambda-header))))
 
 
+;; TBD: change lambda to aut-lambda in exceptions in the following procedure.
 (define (xlat-kw-procedure-aut compiler symtbl toplevel? expr-head expr-tail)
   (dwl4 "xlat-kw-procedure-aut")
   (assert (hrecord-is-instance? compiler <compiler>))
@@ -395,6 +401,7 @@
 	  (let ((t-gen-proc (get-symbol symtbl name))
 		(binder (compiler-get-binder compiler)))
 	    (cond
+	     ;; TBD: change eqv? to eq?.
 	     ((eqv? t-gen-proc #f)
 	      (car (translate-define-gen-proc compiler symtbl name)))
 	     ((not (is-t-instance?
@@ -409,6 +416,7 @@
   (dwl4 "declare-method ENTER")
   (assert (hrecord-is-instance? compiler <compiler>))
   (assert (is-target-object? r-gen-proc))
+  ;; TBD: Do we need strong-assert here?
   (assert (or (is-tc-simple-proc? r-method-type)
 	      (is-tc-param-proc? r-method-type)))
   (dwl4 "declare-method/1")
@@ -464,6 +472,7 @@
 	    (let* ((t-gen-proc0 (get-symbol symtbl s-gen-proc))
 		   (p-gen-proc
 		    (cond
+		     ;; TBD: change eqv? to eq?.
 		     ((eqv? t-gen-proc0 #f)
 		      (translate-define-gen-proc compiler symtbl s-gen-proc))
 		     ((and (is-entity? t-gen-proc0)
@@ -565,6 +574,7 @@
 	       (to-new2
 		(add-method-to-generic2!
 		 binder t-gen-proc decl to-new1))
+	       ;; TBD: change eqv? to eq?.
 	       (declared? (not (eqv? to-new1 to-new2))))
 	  (dwl2 "do-add-method/1")
 	  (dwl2 declared?)
@@ -614,6 +624,7 @@
 	    (s-method (cadr expr-tail)))
 	(dwl2 "xlat-kw-add-method/2")
 	(dwl2 s-gen-proc)
+	;; TBD: Should we raise an exception if s-gen-proc is not a symbol?
 	(hfield-set! compiler 's-cur-toplevel-def s-gen-proc)
 	(let* ((t-gen-proc0
 		(if (symbol? s-gen-proc) (get-symbol symtbl s-gen-proc) #f))
@@ -785,6 +796,7 @@
 		    (body (drop expr-tail 1))
 		    (translate (lambda (expr)
 				 (translate-expr compiler symtbl #f expr))))
+		;; TBD: change this to strong-assert.
 		(assert (not-null? condition))
 		(let ((t-condition (translate condition))
 		      (t-result
@@ -802,6 +814,7 @@
 			  (target-type=? (get-entity-type t-condition)
 					 tc-boolean))
 		      (let ((t-type (get-entity-type t-result))
+			    ;; TBD: Should we check only t-result here? 
 			    (type-dispatched?
 			     (and
 			      (entity-type-dispatched? t-condition)
@@ -812,6 +825,8 @@
 			      (entity-always-returns? t-condition)
 			      (entity-always-returns? t-result)
 			      (entity-always-returns? t-body)))
+			    ;; It is possible that the body is not evaluated
+			    ;; at all. So we don't check it in the following.
 			    (never-returns?
 			     (or (entity-never-returns? t-condition)
 				 (entity-never-returns? t-result)))
@@ -829,7 +844,9 @@
 				      pure?
 				      #f
 				      (hfield-ref compiler 'inside-param-def?)
-				      (get-entity-type t-result)
+				      ;; The following expression has been
+				      ;; fixed.
+				      (get-entity-value t-result)
 				      always-returns?
 				      never-returns?
 				      t-condition
@@ -1242,6 +1259,7 @@
   (assert (list? expr-tail))
   (check-toplevel toplevel?)
   (if (not (= (length expr-tail) 3))
+      ;; TBD: change the following exception name
       (raise 'def-param-proc-syntax-error)
       (let ((s-name (car expr-tail))
 	    (s-typevars (cadr expr-tail))
@@ -1290,57 +1308,57 @@
 		 var
 		 r-proc
 		 #t))
-		(let* ((binder (compiler-get-binder compiler))
-		       (address (compiler-alloc-loc compiler s-name #t))
-		       (to-decl
-			(make-target-object
-			 tc-object #t #f address
-			 #f #t #f '())))
-		  (add-symbol! symtbl s-name to-decl)
-		  (let* ((local-env (make-environment symtbl type-params))
-			 (r-proc0 (translate-expr compiler local-env #f s-proc))
-			 (inst-type (get-entity-type r-proc0))
-			 (to-param-proc-class
-			  (make-param-proc-class-object
-			   "instance of :param-proc"
-			   r-type-params
-			   inst-type))
-			 (to-new (make-param-proc-object
-				  s-name
-				  to-param-proc-class
-				  r-proc0
-				  address))
-			 (r-proc (make-param-proc2 r-type-params
-						   to-param-proc-class
-						   r-proc0 'toplevel '()
-						   to-new)))
-		    (set-object1! to-decl to-new)
-		    (let ((var (make-object-var to-decl)))
-		      (compiler-set-fixed-tvars! compiler old-fixed-tvars)
-		      (hfield-set! compiler 'inside-param-def?
-				   inside-param-def-old?)
-		      (list
-		       (make-hrecord <forward-declaration>
-				     tt-none
-				     #t
-				     #t
-				     '()
-				     
-				     ;; pure? has probably no effect here
-				     #t
-				     #f
-				     #f
-				     '()
-				     
-				     var
-				     to-param-proc-class
-				     #f
-				     #f)
-		       (make-normal-var-def
-			to-param-proc-class
-			var
-			r-proc
-			#t))))))))))
+	      (let* ((binder (compiler-get-binder compiler))
+		     (address (compiler-alloc-loc compiler s-name #t))
+		     (to-decl
+		      (make-target-object
+		       tc-object #t #f address
+		       #f #t #f '())))
+		(add-symbol! symtbl s-name to-decl)
+		(let* ((local-env (make-environment symtbl type-params))
+		       (r-proc0 (translate-expr compiler local-env #f s-proc))
+		       (inst-type (get-entity-type r-proc0))
+		       (to-param-proc-class
+			(make-param-proc-class-object
+			 "instance of :param-proc"
+			 r-type-params
+			 inst-type))
+		       (to-new (make-param-proc-object
+				s-name
+				to-param-proc-class
+				r-proc0
+				address))
+		       (r-proc (make-param-proc2 r-type-params
+						 to-param-proc-class
+						 r-proc0 'toplevel '()
+						 to-new)))
+		  (set-object1! to-decl to-new)
+		  (let ((var (make-object-var to-decl)))
+		    (compiler-set-fixed-tvars! compiler old-fixed-tvars)
+		    (hfield-set! compiler 'inside-param-def?
+				 inside-param-def-old?)
+		    (list
+		     (make-hrecord <forward-declaration>
+				   tt-none
+				   #t
+				   #t
+				   '()
+				   
+				   ;; pure? has probably no effect here
+				   #t
+				   #f
+				   #f
+				   '()
+				   
+				   var
+				   to-param-proc-class
+				   #f
+				   #f)
+		     (make-normal-var-def
+		      to-param-proc-class
+		      var
+		      r-proc
+		      #t))))))))))
 
 
 (define (xlat-kw-param-proc compiler symtbl toplevel? expr-head expr-tail)
@@ -1645,6 +1663,7 @@
 		(hfield-ref r-var-expr 'variable))
 	       ((is-target-object? r-var-expr)
 		r-var-expr)
+	       ;; TBD: change "undefined" to "invalid" (?)
 	       (else (raise (list 'reexport:undefined-variable s-var)))))) 
 	;; The order of reexports should not matter.
 	(hfield-set! compiler 'reexports
@@ -1713,6 +1732,7 @@
 	      ;; (dvar2-set! to-decl)
 	      ;; (raise 'stop-new)
 
+	      ;; It is probably unnecessary to wrap the definition in a list.
 	      (list
 	       (make-hrecord
 		<signature-definition>
@@ -1745,6 +1765,7 @@
   (assert (hrecord-is-instance? expr-head <keyword-define-param-signature>))
   (check-toplevel toplevel?)
   (if (not (and (list? expr-tail) (>= (length expr-tail) 3)))
+      ;; TBD: Change "signature" to "param-signature"
       (raise 'invalid-signature-definition))
   (let ((s-name (car expr-tail))
 	(s-type-vars (cadr expr-tail))
@@ -1860,33 +1881,6 @@
       (raise 'zero:invalid-number-of-arguments)))
 
 
-(define (make-prim-class-proc compiler symtbl desc checked?)
-  (dwl4 "make-prim-class-proc")
-  (strong-assert (and (list? desc) (= (length desc) 3)))
-  (let ((theme-name (car desc))
-	(target-name (cadr desc))
-	(s-type (caddr desc)))
-    (strong-assert (symbol? theme-name))
-    (strong-assert (symbol? target-name))
-    (let* ((bound-value (get-symbol symtbl theme-name))
-	   (rebind? (check-existing-binding? compiler bound-value))
-	   (r-type (translate-expr compiler symtbl #f s-type))
-	   (r-proc
-	    (if checked?
-		(get-checked-prim-proc target-name r-type)
-		(get-prim-proc-expression target-name r-type)))
-	   (var
-	    (bind-variable-with-object!
-	     compiler symtbl rebind? #t theme-name r-type
-	     (get-entity-value r-proc)))
-	   (def (make-normal-var-def
-		 r-type
-		 var
-		 r-proc
-		 rebind?)))
-      def)))
-
-
 (define (symbol-or-null? x)
   (or (symbol? x) (null? x)))
 
@@ -1924,9 +1918,11 @@
 	  (symbol-or-null? s-equal-objects-pred)
 	  (symbol-or-null? s-equal-contents-pred)))
     (raise (list 'invalid-equality-predicate name)))
+   ;; TBD: change eqv? to eq?.
    ((and (not goops?) (not (eqv? superclass tc-object)))
     (raise 'invalid-prim-class-superclass-2))
    ((and goops?
+	 ;; TBD: change eqv? to eq?.
 	 (not (eqv? superclass tc-object))
 	 (not (tno-field-ref superclass 'goops?)))
     (raise 'invalid-goops-class-superclass))
@@ -1934,12 +1930,14 @@
     (raise 'noninheritable-superclass))
    ((and (not goops?) (null? s-member-pred))
     (raise 'no-membership-predicate))
+   ;; TBD: Check that target-name is nonnull for goops classes.
    (else
     (let* ((ht (if goops? 
 		   (hfield-ref compiler 'ht-goops-classes)
 		   (hfield-ref compiler 'ht-prim-classes)))
 	   (s-search (if goops? target-name s-member-pred))
 	   (x (hashq-ref ht s-search)))
+      ;; TBD: Change eqv? to eq?.
       (if (not (eqv? x #f))
 	  (begin
 	    (if goops?
@@ -2031,6 +2029,7 @@
 	      (dwl4 "translate-define-prim-class EXIT")
 	      (if (not rebind?)
 		  (list decl result)
+		  ;; TBD: remove "list" from the following.
 		  (list result)))))))))
 
 
@@ -2145,6 +2144,7 @@
     (list var expr-type expr #f)))
 
 
+;; TBD: Check that no proper clauses begin with "else".
 (define (translate-match-type compiler symtbl toplevel?
 			      expr-head expr-tail strong?)
   (assert (hrecord-is-instance? compiler <compiler>))
@@ -2209,6 +2209,7 @@
 
 (define (xlat-kw-static-type-of compiler symtbl toplevel?
 				expr-head expr-tail)
+  ;; Should we compute the component expression in the translated code?
   (if (= (length expr-tail) 1)
       (let* ((sexpr (car expr-tail))
 	     (ent (translate-expr compiler symtbl toplevel? sexpr)))
@@ -2264,6 +2265,7 @@
 			    address x-handler))))))
 
 
+;; TBD: Remove $'s from the exception names.
 (define (xlat-kw-module-ref compiler symtbl toplevel? expr-head expr-tail)
   (if (= (length expr-tail) 2)
       (let ((l-module (car expr-tail))
@@ -2544,6 +2546,7 @@
 
 (define (theme-do-parse-unit compiler subexprs unit-type)
   (dwl4 "theme-do-parse-unit")
+  ;; TBD: Remove the following assertion and move the test for length.
   (strong-assert (>= (length subexprs) 2))
   (let* ((name0 (cadr subexprs))
 	 (name (if (symbol? name0) (list name0) name0)))
@@ -2578,7 +2581,9 @@
 	      (hfield-set! compiler 'state 'importing)
 	      (hfield-set! compiler 'env env1)
 	      (hfield-set! compiler 'env-all env2)
+	      ;; TBD: Remove the following statement.
 	      (hfield-set! compiler 'imports imports)
+	      ;; TBD: Remove the following statement.
 	      (hfield-set! compiler 'imports-with-reexports
 			   imports-with-reexports)
 	      (hfield-set! (hfield-ref compiler 'binder)
