@@ -255,12 +255,15 @@
 		      pretty-print?)
 	(newline target-output-file)
 	(newline target-output-file)
-	(write-target (cons 'prelink-body
-			    (hfield-ref compiler 'l-prelinked-bodies))
-		      target-output-file
-		      pretty-print?)
-	(newline target-output-file)
-	(newline target-output-file)
+	;; prelink-body forms are not allowed in interfaces.
+	(if (not (eq? unit-type1 'interface))
+	    (begin
+	      (write-target (cons 'prelink-body
+				  (hfield-ref compiler 'l-prelinked-bodies))
+			    target-output-file
+			    pretty-print?)
+	      (newline target-output-file)
+	      (newline target-output-file)))
 	(for-each (lambda (expr)
 		    (write-target expr target-output-file pretty-print?)
 		    (newline target-output-file)
@@ -293,7 +296,6 @@
   (init-compiler compiler module-search-path
 		 #f no-expansion?
 		 backtrace? err-expr? pretty-print?)
-  (set! gl-binder (hfield-ref compiler 'binder))
   (let* ((unit-type1 (if (null? unit-type)
 			 (get-source-unit-type source-filename)
 			 unit-type))
