@@ -79,10 +79,11 @@
 (define address-number-keyword -3)
 
 
-(define keyword-address
-  (make-hrecord <address> '() address-number-keyword '() #t))
-
+(define (make-keyword-address s-kw)
+  (assert (symbol? s-kw))
+  (make-hrecord <address> '() address-number-keyword s-kw #t))
 
+  
 (define (check-address? address)
   (let ((module (hfield-ref address 'module))
 	(number (hfield-ref address 'number))
@@ -97,13 +98,12 @@
 	    (or (null? source-name) (symbol? source-name))))
 	  ((= number address-number-builtin)
 	   (and
-	    (eqv? module 'builtins)
+	    (eq? module 'builtins)
 	    (symbol? source-name)))
 	  ((= number address-number-target)
 	   (and (null? module) (not-null? source-name)))
 	  ((= number address-number-keyword)
-	   (and (eqv? module '())
-		(eqv? source-name '())))
+	   (eq? module '()))
 	  (else (raise 'invalid-address-number))))))
 
 
@@ -116,12 +116,13 @@
      
 (define (address=? addr1 addr2)
   (and
-   (equal? (hfield-ref addr1 'module) (hfield-ref addr2 'module))
-   (= (hfield-ref addr1 'number) (hfield-ref addr2 'number))
-   (eqv? (hfield-ref addr1 'source-name) (hfield-ref addr2 'source-name))
+   ;; Formerly there was "eqv?" here.
+   (eq? (hfield-ref addr1 'source-name) (hfield-ref addr2 'source-name))
    ;; All address numbers should be unique within same module
    ;; so the following test may not be necessary.
-   (eq? (hfield-ref addr1 'toplevel?) (hfield-ref addr2 'toplevel?))))
+   (eq? (hfield-ref addr1 'toplevel?) (hfield-ref addr2 'toplevel?))
+   (= (hfield-ref addr1 'number) (hfield-ref addr2 'number))
+   (equal? (hfield-ref addr1 'module) (hfield-ref addr2 'module))))
 
 
 (define (alloc-builtin-loc s-name)
@@ -483,6 +484,7 @@
 
 
 (define (is-known-pure-entity? ent)
+  ;; TBD: if ent is an expression check that it is pure.
   (let ((to (get-entity-value ent)))
     (and (not-null? to) (is-known-object? to))))
 
