@@ -379,7 +379,7 @@
 						inside-param-def?
 						runtime-arglist-typecheck?)
   (if (and (not-null? arg-list)
-	   (or-map? entity-type-is-none? arg-list))
+	   (contains-none-value? binder arg-list))
       (raise (list
 	      'procedure-argument-with-type-none
 	      (cons 's-proc-name
@@ -528,7 +528,7 @@
 						always-returns?
 						never-returns?
 						l-default-params)
-  (dwl1 "translate-param-proc-appl-inside-param")
+  (dwl2 "translate-param-proc-appl-inside-param")
   (assert (is-binder? binder))
   (assert (is-entity? r-inst-type))
   (assert (and (list? r-type-var-values)
@@ -585,7 +585,7 @@
 				    always-returns?
 				    never-returns?
 				    type-check?)
-  (dwl1 "make-general-gen-proc-appl")
+  (dwl2 "make-general-gen-proc-appl")
   (assert (is-binder? binder))
   (assert (is-entity? gen-proc))
   (assert (and (list? arglist)
@@ -862,7 +862,7 @@
   (assert (and (list? l-default-params)
 	       (and-map? is-entity? l-default-params)))
   (if (and (not-null? arglist)
-	   (or-map? entity-type-is-none? arglist))
+	   (contains-none-value? binder arglist))
       (raise (list
 	      'procedure-argument-with-type-none
 	      (cons 's-proc-name
@@ -995,7 +995,7 @@
   (assert (and (list? arglist)
 	       (and-map? is-entity? arglist)))
   (if (and (not-null? arglist)
-	   (or-map? entity-type-is-none? arglist))
+	   (contains-none-value? binder arglist))
       (raise (list
 	      'procedure-argument-with-type-none
 	      (cons 's-proc-name
@@ -1198,7 +1198,8 @@
 		  '())))
 
 
-(define (make-general-generic-proc-dispatch generic-proc arg-types proc-type
+(define (make-general-generic-proc-dispatch binder
+					    generic-proc arg-types proc-type
 					    with-result?
 					    appl-pure?
 					    appl-always-returns?
@@ -1215,7 +1216,7 @@
 	  (else
 	   (dvar1-set! proc-type)
 	   (raise 'internal-error)))))
-    (if (and with-result? (entity-is-none? to-result-type))
+    (if (and with-result? (entity-is-none1? binder to-result-type))
 	(raise 'illegal-generic-dispatch-result-none))
     ;; Not sure about type-dispatched? and need-revision? here.
     (make-hrecord <generic-proc-dispatch>
@@ -1240,7 +1241,7 @@
 ;; as argument types
 (define (do-translate-normal-genproc-appl binder
 					  gen-proc arglist arg-types)
-  (dwl1 "do-translate-normal-genproc-appl")
+  (dwl2 "do-translate-normal-genproc-appl")
   (let ((selection (select-best-method
 		    binder
 		    arg-types
@@ -1248,7 +1249,7 @@
     ;; MIETI puhtauden periytyminen metodeilla.
     (cond
      ((null? selection)
-      (dwl1 "do-translate-normal-genproc-appl/1")
+      (dwl2 "do-translate-normal-genproc-appl/1")
       (if (or-map? contains-free-tvars-fwd? arg-types)
 	  (make-general-gen-proc-appl binder gen-proc arglist
 				      tc-object #f #f #f #f #f)
@@ -1260,7 +1261,7 @@
       (let ((best-match (car selection)))
 	(assert (pair? best-match))
 	(let ((mtc (cdr best-match)))
-	  (dwl1 "do-translate-normal-genproc-appl/3")
+	  (dwl2 "do-translate-normal-genproc-appl/3")
 	  (dvar1-set! best-match)
 	  (if (not (is-tc-simple-proc? mtc))
 	      (raise 'invalid-procedure-1)
@@ -1269,7 +1270,7 @@
 		     (and (tno-field-ref mtc 'pure-proc?)
 			  (and-map? is-pure-entity? arglist)))
 		    (method (car best-match)))
-		(dwl1 "do-translate-normal-genproc-appl/4")
+		(dwl2 "do-translate-normal-genproc-appl/4")
 		(if (or (tno-field-ref mtc 'static-method?)
 			(is-exact-match? binder arg-types mtc))
 		    (make-gen-proc-appl-best-match
@@ -1287,7 +1288,7 @@
 		       #t
 		       pure? always-returns? never-returns? #t))))))))
      (else
-      (dwl1 "do-translate-normal-genproc-appl/5")
+      (dwl2 "do-translate-normal-genproc-appl/5")
       ;; Method ambiguity is not a compile time error (?).
       ;; Methods defined after calling the generic procedure
       ;; may remove the (run time) ambiguity.
@@ -1322,7 +1323,7 @@
 
 (define (do-translate-genproc-appl binder
 				   gen-proc arglist arg-types)
-  (dwl1 "do-translate-genproc-appl")
+  (dwl2 "do-translate-genproc-appl")
   (assert (is-binder? binder))
   (strong-assert (is-target-object? gen-proc))
   (assert (list? arglist))
@@ -1337,7 +1338,7 @@
 	(if (null? l-signatures)
 	    (do-translate-normal-genproc-appl binder gen-proc arglist arg-types)
 	    (begin
-	      (dwl1 "do-translate-genproc-appl/2")
+	      (dwl2 "do-translate-genproc-appl/2")
 	      (if (not (contains-free-tvars-general? arglist))
 		  (let* ((p-match (match-call-with-signatures binder
 							      gen-proc
@@ -1385,7 +1386,7 @@
     (raise 'invalid-generic-procedure))
    ((or (not type-check?) (or-map? contains-free-tvars-fwd? arg-types))
     (dwl2 "translate-generic-proc-dispatch EXIT 5")
-    (make-general-generic-proc-dispatch generic-proc arg-types
+    (make-general-generic-proc-dispatch binder generic-proc arg-types
 					(get-general-generic-class
 					 with-result? appl-pure?)
 					with-result?
@@ -1444,6 +1445,7 @@
 	     (else
 	      (dwl2 "translate-generic-proc-dispatch EXIT 2")
 	      (make-general-generic-proc-dispatch
+	       binder
 	       generic-proc arg-types mtc with-result?
 	       appl-pure?
 	       appl-always-returns?
@@ -1485,6 +1487,7 @@
 	      (raise 'invalid-attributes-in-dispatch))
 	  (dwl2 "translate-generic-proc-dispatch EXIT 3")
 	  (make-general-generic-proc-dispatch
+	   binder
 	   generic-proc arg-types proc-type with-result?
 	   appl-pure?
 	   appl-always-returns?
@@ -1492,7 +1495,7 @@
 	   #t #t))))))
    (else
     (dwl2 "translate-generic-proc-dispatch EXIT 4")
-    (make-general-generic-proc-dispatch generic-proc arg-types
+    (make-general-generic-proc-dispatch binder generic-proc arg-types
 					(get-general-generic-class
 					 with-result? appl-pure?)
 					with-result?
@@ -1507,7 +1510,7 @@
   (dwl3 "translate-general-genproc-appl")
   (assert (hrecord-is-instance? binder <binder>))
   (if (and (not-null? arglist)
-	   (or-map? entity-type-is-none? arglist))
+	   (contains-none-value? binder arglist))
       (raise (list 'generic-procedure-argument-with-type-none
 		   (cons 's-proc-name
 			 (let ((address (hfield-ref gen-proc 'address)))
@@ -1543,7 +1546,7 @@
   (assert (boolean? type-check?))
   (cond
    ((and (not-null? arguments)
-	 (or-map? entity-type-is-none? arguments))
+	 (contains-none-value? binder arguments))
     (raise (list
 	    'procedure-argument-with-type-none
 	    (cons 's-proc-name
