@@ -114,7 +114,8 @@
 	   (= (length arguments) 2)
 	   (and-map? is-entity? arguments))
       (begin
-	(check-no-none-arguments arguments 'field-ref)
+	(check-no-none-arguments (compiler-get-binder compiler)
+				 arguments 'field-ref)
 	(let ((obj (car arguments))
 	      (sym-field-name (cadr arguments)))
 	  (do-translate-field-ref-appl compiler obj sym-field-name)))
@@ -207,7 +208,8 @@
 	   (= (length arguments) 3)
 	   (and-map? is-entity? arguments))
       (begin
-	(check-no-none-arguments arguments 'field-set!)
+	(check-no-none-arguments (compiler-get-binder compiler)
+				 arguments 'field-set!)
 	(let ((t-obj-expr (car arguments))
 	      (t-field-expr (cadr arguments))
 	      (t-value-expr (caddr arguments)))
