@@ -248,15 +248,16 @@
   (assert (hrecord-is-instance? compiler <compiler>))
   (assert (hrecord-is-instance? address-env <address-environment>))
   (strong-assert (and (list? decl) (memv (length decl) '(3 4))))
-  (let* ((module (get-current-module-name compiler))
-	 (type (parse-repr-fwd compiler address-env (list-ref decl 1)))
-	 (contents (list-ref decl 2))
+  (let* ((type (parse-repr-fwd compiler address-env (list-ref decl 1)))
+	 (ent-contents (parse-repr-fwd compiler address-env (list-ref decl 2)))
 	 (l-opt-contents (if (= (length decl) 4)
 			     (list-ref decl 3)
 			     '()))
-	 (prim-expr (make-primitive-object-w-opt type contents
+	 (prim-expr (make-primitive-object-w-opt type ent-contents
 						 l-opt-contents)))
-    (assert (target-type=? (get-primitive-type contents) type))
+    ;; Maybe we could use equal-types? here.
+    (assert (is-t-subtype? (compiler-get-binder compiler)
+			   (get-entity-type ent-contents) type))
     prim-expr))
 
 
