@@ -109,7 +109,7 @@
        '(set! (toplevel gl-pretty-backtrace-support?) (const #f)))))
 
 	       
-(define (compile-program-prefix-scheme script?)
+(define (compile-program-prefix-scheme linker script?)
   (list
    '(import (guile))
    '(import (srfi srfi-1))
@@ -119,7 +119,9 @@
    (if script?
        '(set! gl-script? #t)
        '(set! gl-script? #f))
-   '(set! gl-pretty-backtrace-support? #f)))
+   (if (hfield-ref linker 'runtime-pretty-backtrace?)
+       '(set! gl-pretty-backtrace-support? #t)
+       '(set! gl-pretty-backtrace-support? #f))))
 
 
 (define (not-null-texpr? texpr)
@@ -161,6 +163,16 @@
   (dwl2 "theme-target-compile-program")
   (if (use-tree-il? linker)
       (map-in-order (lambda (entity)
+
+		      ;; TBR
+		      ;; (set! gl-counter28 (+ gl-counter28 1))
+		      ;; (dwl1 "theme-target-compile-program/1")
+		      ;; (dwl1 gl-counter28)
+		      ;; (if (= gl-counter28 600)
+		      ;; 	  (begin
+		      ;; 	    (dwl1 "HEP600")
+		      ;; 	    (set! gl-flag16? #t)))
+		      
 		      (let ((result (tc-tree-il-entity linker entity)))
 			(if (not-null? result) result '(void))))
 		    lst-entities)
@@ -192,7 +204,7 @@
   (assert (and (list? imports) (and-map? is-module-name? imports)))
   (for-each
    (lambda (submod)
-     (theme-link-unit linker "" submod 'interface address-env '() #f #f))
+     (theme-link-interfaces linker "" submod 'interface address-env '() #f))
    imports))
 
 
@@ -201,33 +213,35 @@
   (assert (hrecord-is-instance? address-env <address-environment>))
   (assert (is-module-name? mod-name))
   (assert (and (list? imports) (and-map? is-module-name? imports)))
-  (theme-link-unit linker "" mod-name 'interface address-env '() #f #f)
   (for-each
    (lambda (submod)
-     (theme-link-unit linker "" submod 'interface address-env '() #f #f))
+     (theme-link-interfaces linker "" submod 'interface address-env
+			    '() #f))
    imports))
 
 
-(define (theme-link-interface-imports linker address-env hierarchy imports)
+(define (theme-link-interface-imports linker address-env mod-name
+				      hierarchy imports)
   (assert (hrecord-is-instance? linker <linker>))
   (assert (hrecord-is-instance? address-env <address-environment>))
   (assert (and (list? hierarchy) (and-map? is-module-name? hierarchy)))
   (assert (and (list? imports) (and-map? is-module-name? imports)))
   (for-each
    (lambda (submod)
-     (theme-link-unit linker "" submod 'interface address-env hierarchy #f #f))
+     (theme-link-interfaces linker "" submod 'interface address-env
+			    hierarchy #f))
    imports))
 
 
-(define (theme-prelink linker address-env hierarchy l-bodies)
-  (assert (hrecord-is-instance? linker <linker>))
-  (assert (hrecord-is-instance? address-env <address-environment>))
-  (assert (and (list? hierarchy) (and-map? is-module-name? hierarchy)))
-  (assert (and (list? l-bodies) (and-map? is-module-name? l-bodies)))
-  (for-each
-   (lambda (submod)
-     (theme-link-unit linker "" submod 'body address-env hierarchy #f #t))
-   l-bodies))
+;; (define (theme-prelink linker address-env hierarchy l-bodies)
+;;   (assert (hrecord-is-instance? linker <linker>))
+;;   (assert (hrecord-is-instance? address-env <address-environment>))
+;;   (assert (and (list? hierarchy) (and-map? is-module-name? hierarchy)))
+;;   (assert (and (list? l-bodies) (and-map? is-module-name? l-bodies)))
+;;   (for-each
+;;    (lambda (submod)
+;;      (theme-link-interfaces linker "" submod 'body address-env hierarchy #f #t))
+;;    l-bodies))
 
 
 ;; The output of this procedure updates necessary declared method variables
@@ -282,9 +296,99 @@
   (filter (lambda (texpr) (not (equal? texpr '(void)))) l-texprs))
 
 
-(define (theme-link-unit linker source-filename mod-name unit-type address-env
-			 hierarchy program? prelink?)
-  (dwl4 "theme-link-unit")
+(define (add-body-to-link! linker mod-name)
+  (let ((l-bodies (hfield-ref linker 'l-bodies-to-link)))
+    (if (not (member mod-name l-bodies))
+	(hfield-set! linker 'l-bodies-to-link (cons mod-name l-bodies)))))
+
+
+(define (add-body-to-process! linker mod-name)
+  (let ((l-bodies (hfield-ref linker 'l-bodies-to-process)))
+    (if (not (member mod-name l-bodies))
+	(hfield-set! linker 'l-bodies-to-process (cons mod-name l-bodies)))))
+
+
+(define (emit-debug-module-start linker mod-name)
+  (let ((expr1 (make-debug-output-expr "starting to load module body "))
+	(expr2 (make-debug-output-expr mod-name))
+	(expr3 (make-debug-output-expr "\n")))
+    (hfield-set! linker 'repr-list
+		 (cons expr3
+		       (cons expr2
+			     (cons expr1
+				   (hfield-ref linker 'repr-list)))))))
+
+
+(define (emit-debug-module-end linker mod-name)
+  (let ((expr1 (make-debug-output-expr "finished loading module body "))
+	(expr2 (make-debug-output-expr mod-name))
+	(expr3 (make-debug-output-expr "\n")))
+    (hfield-set! linker 'repr-list
+		 (cons expr3
+		       (cons expr2
+			     (cons expr1
+				   (hfield-ref linker 'repr-list)))))))
+
+
+(define (theme-link-unit linker mod-name unit-type address-env
+			 prelink? l-visited)
+  (cond
+   ((member mod-name l-visited)
+    (raise (list 'cyclic-prelink (cons 'mod-name mod-name))))
+   ((member mod-name (hfield-ref linker 'l-linked-bodies2))
+    '())
+   (else
+    (let* ((ext (get-pcode-suffix unit-type))
+	   (source-filename
+	    (search-file (hfield-ref linker 'module-search-path)
+			 mod-name ext)))
+      (if (not (eq? source-filename #f))
+	  (let ((l-new-visited (cons mod-name l-visited))
+		(source-file
+		 (theme-open-input-file source-filename)))
+	    (hfield-set! linker 'l-linked-bodies2
+			 (cons mod-name
+			       (hfield-ref linker 'l-linked-bodies2)))
+	    ;; Skip the header except prelinked bodies.
+	    (do ((i 0 (+ i 1))) ((>= i 6)) (theme-read source-file))
+	    (let ((prev-module (hfield-ref linker 'current-module)))
+	      (hfield-set! linker 'current-module mod-name)
+	      (let* ((l-prelink (theme-read source-file))
+		     (body (theme-read-file source-file)))
+		(theme-close-input-port source-file)
+		(assert (and (list? l-prelink)
+			     (and-map? is-module-name? l-prelink)))
+		(if prelink?
+		    (for-each
+		     (lambda (mod-name1)
+		       (theme-link-unit linker mod-name1 'body address-env
+					#t l-new-visited))
+		     l-prelink))
+		(if (not-null? (cdr body)) (raise 'extra-code-in-impl-file))
+		(hfield-set! linker 'state 'pcode-reading)
+		(dw1 "Body pseudocode reading: ")
+		(dwl1 mod-name)
+		(if (hfield-ref linker 'module-debug-output?)
+		    (emit-debug-module-start linker mod-name))
+		(do ((s-body (car body) (cdr s-body)))
+		    ((null? s-body))
+		  (let ((sexpr (car s-body)))
+		    (hfield-set! linker 'current-toplevel-expr sexpr)
+		    (let ((rexpr (theme-read-body-expr linker address-env
+						       sexpr)))
+		      (hfield-set! linker 'current-toplevel-expr '())
+		      (hfield-set! linker 'repr-list
+				   (cons rexpr
+					 (hfield-ref linker 'repr-list)))))))
+	      (hfield-set! linker 'current-module prev-module))
+	    (if (hfield-ref linker 'module-debug-output?)
+		(emit-debug-module-end linker mod-name)))
+	  (raise 'module-not-found))))))
+
+
+(define (theme-link-interfaces linker source-filename mod-name unit-type
+			       address-env hierarchy program?)
+  (dwl4 "theme-link-interfaces")
   (assert (hrecord-is-instance? linker <linker>))
   (assert (string? source-filename))
   ;; Module name has to be a list.
@@ -305,16 +409,16 @@
       (begin
 	;;	(write-error-info mod-name)
 	(raise 'cyclic-interface-dependency)))
-     ((and prelink? (not (eq? unit-type 'body)))
-      (raise 'internal-error-with-prelinking))
-     ((and prelink? (member mod-name hierarchy))
-      (begin
-	;;	(write-error-info mod-name)
-	(raise (list 'cyclic-prelink-dependency
-		     (cons 'module-name mod-name)))))
-     ((and prelink? (not (member mod-name
-				 (hfield-ref linker 'linked-interfaces))))
-      (raise 'prelinked-body-interface-not-linked))
+     ;; ((and prelink? (not (eq? unit-type 'body)))
+     ;;  (raise 'internal-error-with-prelinking))
+     ;; ((and prelink? (member mod-name hierarchy))
+     ;;  (begin
+     ;; 	;;	(write-error-info mod-name)
+     ;; 	(raise (list 'cyclic-prelink-dependency
+     ;; 		     (cons 'module-name mod-name)))))
+     ;; ((and prelink? (not (member mod-name
+     ;; 				 (hfield-ref linker 'linked-interfaces))))
+     ;;  (raise 'prelinked-body-interface-not-linked))
      ((and (eq? unit-type 'interface)
 	   (member mod-name (hfield-ref linker 'linked-interfaces)))
       (begin
@@ -352,10 +456,19 @@
 			      (theme-read source-file)
 			      (theme-read source-file)))
 		     (l-prelinked-bodies
-		      (theme-read source-file)))
+		      (theme-read source-file))
+		     (body (theme-read-file source-file)))
+		(theme-close-input-port source-file)
+		(if (not-null? (cdr body))
+		    (raise 'extra-code-in-impl-file))
 		(dwl4 "theme-do-link/4")
 		(if (not (module-name=? mod-name actual-name))
-		    (raise 'module-name-mismatch))
+		    (begin
+		      (display mod-name)
+		      (newline)
+		      (display actual-name)
+		      (newline)
+		      (raise 'module-name-mismatch)))
 		;; It is essential to update linker-interfaces and linked-bodies
 		;; here before further linking of the unit.
 		(cond
@@ -375,22 +488,26 @@
 		    (theme-link-program-imports linker address-env imports))
 		   ((eq? unit-type 'interface)
 		    (let ((new-hierarchy (cons mod-name hierarchy)))
-		      (theme-link-interface-imports linker address-env new-hierarchy imports)))
+		      (theme-link-interface-imports linker address-env
+						    mod-name new-hierarchy
+						    imports)))
 		   ((eq? unit-type 'body)
 		    (theme-link-body-imports linker address-env mod-name imports))
 		   (else (raise 'internal-error)))
-		  (if (not-null? l-prelinked-bodies)
-		      (if (not (eq? unit-type 'interface))
-			  (begin
-			    (dw1 "Prelinking bodies: ")
-			    (dwl1 mod-name)
-			    (theme-prelink linker address-env visited-cur
-					   l-prelinked-bodies))
-			  (raise 'prelink-in-interface)))
+		  ;; (if (not-null? l-prelinked-bodies)
+		  ;;     (if (not (eq? unit-type 'interface))
+		  ;; 	  (begin
+		  ;; 	    (dw1 "Prelinking bodies: ")
+		  ;; 	    (dwl1 mod-name)
+		  ;; 	    (theme-prelink linker address-env visited-cur
+		  ;; 			   l-prelinked-bodies))
+		  ;; 	  (raise 'prelink-in-interface)))
 
 		  (dw1 "Reading unit contents: ") 
-		  (dwl1 mod-name)
-
+		  (dw1 mod-name)
+		  (dw1 ", ")
+		  (dwl1 unit-type)
+		  
 		  ;; Updating variable numbering.
 		  (let ((ht (hfield-ref linker 'ht-module-indices)))
 		    ;; An interface is always read before its body
@@ -412,47 +529,69 @@
 				i-number-of-vars))
 			    (hash-set! ht actual-name i-current-var-start))))
 
-		  (let ((body
-			 (theme-read-file source-file)))
-		    (theme-close-input-port source-file)
-		    (if (not-null? (cdr body)) (raise 'extra-code-in-impl-file))
-		    (hfield-set! linker 'state 'target-compilation)
-		    (dw1 "Target compilation: ")
-		    (dw1 mod-name)
-		    (dw1 ", ")
-		    (dwl1 unit-type)
+		  (if (eq? unit-type 'interface)
+		      (add-body-to-process! linker mod-name))
+		  (cond
+		   ;; ((eq? unit-type 'body)
+		   ;;  (add-body-to-link! linker mod-name)
+		   ;;  (for-each
+		   ;;   (lambda (mod-name1)
+		   ;;     (add-body-to-link! linker mod-name1))
+		   ;;   imports))
+		   ((eq? unit-type 'interface)
+		    (hfield-set! linker 'state 'pcode-reading)
+		    (dw1 "Interface pseudocode reading: ")
+		    (dwl1 mod-name)
 		    (do ((s-body (car body) (cdr s-body)))
 			((null? s-body))
 		      (let ((sexpr (car s-body)))
-;;			(dw1 "compiling pcode expression: ")
-;;			(dwl4 sexpr)
 			(hfield-set! linker 'current-toplevel-expr sexpr)
-			(let ((rexpr (theme-read-body-expr linker address-env sexpr)))
+			(let ((rexpr (theme-read-body-expr linker
+							   address-env
+							   sexpr)))
 			  (hfield-set! linker 'current-toplevel-expr '())
-			  (hfield-set! linker 'repr-list
-				       (cons rexpr (hfield-ref linker 'repr-list))))))
-			  ;; We should change the linking phase here.
-			  ;; Field current-toplevel-expr should also retain
-			  ;; its value during target compilation.
-			  ;; (let ((texprs (theme-target-compile-body-repr linker rexpr)))
-			  ;;   ;; texprs is a list consisting of target expressions
-			  ;;   (guard (exc (else
-			  ;; 		 (make-file-exception
-			  ;; 		  'error-writing-file
-			  ;; 		  (hfield-ref linker 'target-filename))))
-			  ;; 	   (write-module-body
-			  ;; 	    (hfield-ref linker 'target-file) texprs
-			  ;; 	    (hfield-ref linker 'pretty-print?)))))))
-		    (for-each
-		     (lambda (imp)
-		       (if (not (member imp (hfield-ref linker 'linked-bodies)))
-			   (theme-link-unit linker "" imp 'body address-env
-					    '() #f #f)))
-		     imports)))))
+			  (hfield-set!
+			   linker 'repr-list
+			   (cons rexpr
+				 (hfield-ref linker 'repr-list)))))))))))
+	    ;; (for-each
+	    ;;  (lambda (imp)
+	    ;;    (if (not (member imp
+	    ;; 		    (hfield-ref linker 'linked-bodies)))
+	    ;;        (theme-link-interfaces linker "" imp 'body address-env
+	    ;; 			'() #f #f)))
+	    ;;  imports))))))
 	    (raise 'module-not-found)))))
     (hfield-set! linker 'current-module prev-module)))
 
 
+(define (theme-do-link-program linker source-filename mod-name unit-type
+			       address-env)
+  (dwl1 "Linking interfaces")
+  (theme-link-interfaces linker source-filename mod-name unit-type
+			 address-env '() #t)
+  (dwl1 "Linking interfaces/2")
+  (do () ((null? (hfield-ref linker 'l-bodies-to-process)))
+    (let ((mn-cur-body (car (hfield-ref linker 'l-bodies-to-process))))
+      (hfield-set! linker 'l-bodies-to-process
+		   (cdr (hfield-ref linker 'l-bodies-to-process)))
+      (if (not (member mn-cur-body (hfield-ref linker 'linked-bodies)))
+	  (begin
+	    (add-body-to-link! linker mn-cur-body)
+	    (theme-link-interfaces linker "" mn-cur-body
+				   'body address-env '() #f)))))
+  (dwl1 "Linking bodies")
+  (hfield-set! linker 'l-linked-bodies2 '())
+  (for-each
+   (lambda (body-name)
+     (theme-link-unit linker body-name 'body address-env #t ''()))
+   (hfield-ref linker 'l-bodies-to-link))
+  (dwl1 "Reading the main program")
+  ;; No need to prelink the modules imported by the main program
+  ;; since the main program is linked last.
+  (theme-link-unit linker mod-name unit-type address-env #f '()))
+
+
 (define gl-linker
   (make-hrecord <linker>
 		'()
@@ -465,6 +604,7 @@
 		#f
 		#t
 		#t
+		#f
 		'tree-il-2.2
 		'()
 		'()
@@ -503,6 +643,9 @@
 		'()
 		'()
 		'()
+		'()
+		'()
+		'()
 		#f
 		#f
 		'()
@@ -527,6 +670,7 @@
 		     verbose-unlinked-procedures?
 		     backtrace?
 		     runtime-pretty-backtrace?
+		     module-debug-output?
 		     s-intermediate-language)
   (assert (hrecord-is-instance? linker <linker>))
   (assert (hrecord-is-instance? compiler-root-env <environment>))
@@ -537,6 +681,8 @@
   (assert (boolean? verbose-errors?))
   (assert (boolean? verbose-typechecks?))
   (assert (boolean? backtrace?))
+  (assert (boolean? runtime-pretty-backtrace?))
+  (assert (boolean? module-debug-output?))
   (assert (memq s-intermediate-language gl-l-all-int-lang))
   (let* ((linker-root-env (make-address-environment compiler-root-env))
 	 (alloc-loc (lambda (s-name toplevel?)
@@ -631,6 +777,7 @@
     (hfield-set! linker 'strip? strip?)
     (hfield-set! linker 'verbose-unlinked-procedures?
 		 verbose-unlinked-procedures?)
+    (hfield-set! linker 'module-debug-output? module-debug-output?)
     (hfield-set! linker 's-intermediate-language
 		 s-intermediate-language)
     (hfield-set! linker 'repr-list '())
@@ -740,7 +887,8 @@
 			    verbose-typechecks?
 			    verbose-unlinked-procedures?
 			    backtrace?
-			    runtime-pretty-backtrace?)
+			    runtime-pretty-backtrace?
+			    module-debug-output?)
   (assert (hrecord-is-instance? linker <linker>))
   (assert (or (symbol? program-name) (is-module-name? program-name)))
   (assert (and (string? source-filename) (not (string-null? source-filename))))
@@ -756,6 +904,7 @@
   (assert (boolean? verbose-unlinked-procedures?))
   (assert (boolean? backtrace?))
   (assert (boolean? runtime-pretty-backtrace?))
+  (assert (boolean? module-debug-output?))
   (assert (memq s-intermediate-language gl-l-all-int-lang))
   (dw1 "Linking program ")
   (dwl1 program-name)
@@ -788,25 +937,27 @@
 		  verbose-unlinked-procedures?
 		  backtrace?
 		  runtime-pretty-backtrace?
+		  module-debug-output?
 		  s-intermediate-language)
-     (set! gl-binder (hfield-ref linker 'binder-parsing))
      (hfield-set! linker 'interm-file interm-file)
      (hfield-set! linker 'interm-filename interm-filename1)
      (hfield-set! linker 'target-filename target-filename1)
      (let ((l-prefix
 	    (if (use-tree-il? linker)
 		(compile-program-prefix-tree-il linker script?)
-		(compile-program-prefix-scheme script?))))
+		(compile-program-prefix-scheme linker script?))))
        (set! l-comp l-prefix))
      (hfield-set! linker 'state 'pcode-reading)
-     (theme-link-unit linker source-filename program-name
-		      (if script? 'script 'proper-program)
-		      (hfield-ref linker 'root-env) '() #t #f)
+     (theme-do-link-program linker source-filename program-name
+			    (if script? 'script 'proper-program)
+			    (hfield-ref linker 'root-env))
      (if (and (not script?)
 	      (not (get-main linker)))
-	 (raise 'main-procedure-not-defined))
+	 (raise 'main-procedure-not-defined)) 
+     (hfield-set! linker 'state 'decl-param-proc-instances)    
      (set! l-comp
 	   (append l-comp (compile-decl-param-proc-instances linker)))
+     (hfield-set! linker 'state 'making-extra-initializations)
      (set! l-comp
 	   (append
 	    l-comp
@@ -821,7 +972,6 @@
 	       (determine-total-coverage linker)
 	       (do-prevent-stripping linker))))
        (hfield-set! linker 'state 'instantiation)
-       (set! gl-binder (hfield-ref linker 'binder-instantiation))
        (dwl1 "Computing parametrized entity instances")
        (dw1 "Number of expressions to handle: ")
        (dwl1 (length (hfield-ref linker 'repr-list)))
@@ -836,16 +986,44 @@
 	 (set! l-comp
 	       (append l-comp
 		       (compile-used-method-decls linker)))))
+
+     ;; TBR ?
+     (dwl1 "Linked the target entities")
+     
      (if (not script?)
 	 (set! l-comp (append l-comp (list (compile-main linker)))))
+
+     ;; TBR
+     (for-each
+      (lambda (sx)
+	(set! gl-counter24 (+ gl-counter24 1))
+	;; (dwl1 gl-counter24)
+	(if (not (is-primitive-value? sx))
+	    (begin
+	      (display "Illegal target expression detected.\n")
+	      (display gl-counter24)
+	      (newline)
+	      (dvar1-set! sx)
+	      (raise 'stop-illegal-expr))))
+      l-comp)
+     
+     (dwl1 "Making the intermediate file")
+
      (if (use-tree-il? linker)
 	 (let ((l-comp2
 		(tc-tree-il-compile-sequence
 		 linker
 		 (filter-void (handle-tree-il-splices linker l-comp)))))
 	   (if pretty-print?
-	       (pretty-print l-comp2 interm-file)
-;;	       (pretty-print l-comp interm-file)
+	       (begin
+
+		 ;; TBR
+		 ;; (set! gl-counter26 (+ gl-counter26 1))
+		 ;; (dwl1 gl-counter26)
+		 ;; (if (not (is-primitive-value? l-comp2))
+		 ;;     (display "Illegal target expression detected/2.\n"))
+		 
+		 (pretty-print l-comp2 interm-file))
 	       (write l-comp2 interm-file))
 	   (newline interm-file))
 	 (for-each (lambda (texpr)
@@ -908,7 +1086,8 @@
 			     verbose-errors?
 			     verbose-typechecks?
 			     verbose-unlinked-procedures?
-			     runtime-pretty-backtrace?)
+			     runtime-pretty-backtrace?
+			     module-debug-output?)
   (assert (or (symbol? main-module) (is-module-name? main-module)))
   (dwl3 "theme-link-program1")
   (dwl3 strip?)
@@ -939,5 +1118,6 @@
 			      verbose-typechecks?
 			      verbose-unlinked-procedures?
 			      #t
-			      runtime-pretty-backtrace?))
+			      runtime-pretty-backtrace?
+			      module-debug-output?))
 	(raise 'program-file-not-found))))
