@@ -196,6 +196,18 @@
    (else #f)))
 
 
+(define (get-pcode-toplevel-expr-text expr)
+  (cond
+   ((null? expr) #f)
+   ((and (list? expr) (>= (length expr) 1))
+    (if (and (eq? (car expr) 'general-variable)
+	     (>= (length expr) 2))
+	(string-append "variable " (symbol->string (cadr expr)))
+	(string-append "expression of type "
+		       (symbol->string (car expr)))))
+   (else #f)))
+
+
 (define (get-instance-text instance)
   (case (car instance)
     ((class) "an instance of a parametrized class")
@@ -218,23 +230,27 @@
 	  (whole-expr-text
 	   (if (eqv? expr toplevel-expr)
 	       (if (not-null? expr)
-		   (let ((toplevel-expr-text (get-expr-text toplevel-expr)))
-		     (string-append
-		      "while reading toplevel expression of type "
-		      toplevel-expr-text))
+		   (let ((toplevel-expr-text
+			  (get-pcode-toplevel-expr-text toplevel-expr)))
+		     (if toplevel-expr-text
+			 (string-append
+			  "while reading toplevel "
+			  toplevel-expr-text)
+			 "while reading"))
 		   "while reading")
 	       (let ((expr-text (get-expr-text expr))
-		     (toplevel-expr-text (get-expr-text toplevel-expr)))
+		     (toplevel-expr-text
+		      (get-pcode-toplevel-expr-text toplevel-expr)))
 		 (cond
 		  ((and expr-text toplevel-expr-text)
 		   (string-append
 		    "while reading expression of type "
 		    expr-text
-		    " from toplevel expression of type "
+		    " from toplevel "
 		    toplevel-expr-text))
 		  ((and (not expr-text) toplevel-expr-text)
 		   (string-append
-		    "while reading toplevel expression of type "
+		    "while reading toplevel "
 		    toplevel-expr-text))
 		  ((and expr-text (not toplevel-expr-text))
 		   (string-append
@@ -441,6 +457,17 @@
 
 (define (handle-linker-error linker exc)
   (delete-target-files linker)
+  ;; (display "module: ")
+  ;; (display (hfield-ref linker 'current-module))
+  ;; (newline)
+  ;; (display "expression: ")
+  ;; (display (hfield-ref linker 'current-expr))
+  ;; (newline)
+  ;; (display "toplevel expression: ")
+  ;; (display (hfield-ref linker 'current-toplevel-expr))
+  ;; (newline)
+  ;; (display (hfield-ref linker 'state))
+  ;; (newline)
   (cond
    ((and (condition? exc) (theme-file-exception? exc))
     (display (get-file-error-message exc))
