@@ -22,6 +22,9 @@
 	  is-target-object?
 	  get-entity-type
 	  is-t-class?
+	  is-tc-param-proc?
+	  is-t-param-class-instance?
+	  tpc-param-proc
 	  tno-field-ref
 	  theme-link-program
 	  gl-linker
