@@ -2226,7 +2226,7 @@
 	      (car result))))
       (dwl2 "inst-bind-let-expression/5")
       (dwl2 (length type-vars))
-      (if (not (check-no-none-types?
+      (if (not (check-no-none-types? binder
 		(map get-entity-type l-new-bindings)))
 	  (raise 'let:type-none))
 ;;      (if (not (check-no-none-types?
@@ -2239,8 +2239,8 @@
 	    (raise (list 'let:initializer-none
 			 (cons 's-name s-name-none)))))
       (dwl2 "inst-bind-let-expression/6")
-      (set! gl-counter27 (+ gl-counter27 1))
-      (dwl2 gl-counter27)
+      ;; (set! gl-counter27 (+ gl-counter27 1))
+      ;; (dwl2 gl-counter27)
       (let* ((old-body (hfield-ref repr 'body))
 	     (l-subst (inst-make-let-subst old-bindings l-new-bindings))
 	     (new-body
@@ -2253,8 +2253,16 @@
 	     (final-init-exprs-pure?
 	      (and-map? is-pure-entity? l-new-init-exprs))
 	     (let-expr-pure? (and new-body-pure? final-init-exprs-pure?))
+	     (always-returns?
+	      (and (entity-always-returns? new-body)
+		   (and-map? entity-always-returns? l-new-init-exprs)))
+	     (never-returns?
+	      (or (entity-never-returns? new-body)
+		  (or-map? entity-never-returns? l-new-init-exprs)))
 	     (result
 	      (make-hrecord <let-expression>
+			    ;; The type of a let expression is the type of
+			    ;; its body.
 			    (get-entity-type new-body)
 			    (entity-type-dispatched? new-body)
 			    (hfield-ref new-body 'exact-type?)
@@ -2265,10 +2273,8 @@
 			    #f
 			    (get-entity-value new-body)
 
-			    ;; It should be correct to take always-returns? and
-			    ;; never-returns? from the old representation.
-			    (hfield-ref repr 'always-returns?)
-			    (hfield-ref repr 'never-returns?)
+			    always-returns?
+			    never-returns?
 
 			    readonly-bindings?
 			    recursive? order?
