@@ -31,15 +31,6 @@
   (dwl3 (current-time time-utc)))
 
 
-;; (define (theme-compile-var-def compiler env value-expr)
-;;   (assert (hrecord-is-instance? compiler <compiler>))
-;;   (assert (hrecord-is-instance? env <environment>))
-;;   (assert (is-entity? value-expr))
-;;   (if (is-target-object? value-expr)
-;;       (theme-compile-object0-fwd compiler env value-expr '() #t)
-;;       (theme-compile-repr-fwd compiler env value-expr)))
-
-
 ;; This is not for toplevel variables.
 (define (theme-compile-variable compiler env var)
   (assert (hrecord-is-instance? compiler <compiler>))
@@ -48,20 +39,6 @@
   (dwl2 "theme-compile-variable/1")
   (assert (hrecord-is-instance? var <normal-variable>))
   (dwl2 "theme-compile-variable/2")
-
-  ;; TBR
-  ;; (if gl-flag11?
-  ;;     (begin
-  ;;  	(dvar1-set! env)
-  ;;  	(dvar2-set! var)
-  ;;  	(raise 'stop11)))
-  ;; (if (eq? (hfield-ref (hfield-ref var 'address) 'source-name)
-  ;; 	   'args-1347)
-  ;;      (begin
-  ;;   	(dvar1-set! env)
-  ;;   	(dvar2-set! var)
-  ;;   	(raise 'stop1347)))
-
   (list 'internal-variable
 	(theme-compile-address (hfield-ref var 'address))
 	(theme-compile-repr compiler env (get-entity-type var))
@@ -92,6 +69,7 @@
 	  #t)))
 
 
+;; TBD: Remove the following procedure.
 (define (theme-compile-variable-with-expr compiler env var)
   (assert (hrecord-is-instance? compiler <compiler>))
   (assert (hrecord-is-instance? env <environment>))
@@ -117,11 +95,13 @@
   (map theme-compile-tvar-address type-vars))
 
 
+;; TBD: Remove the following procedure.
 (define (compile-object-field compiler env field)
   (cons (car field)
 	(theme-compile-repr compiler env (cdr field))))
 
 
+;; TBD: Remove the following procedure.
 (define (compile-object-fields compiler env obj)
   (assert (hrecord-is-instance? compiler <compiler>))
   (assert (hrecord-is-instance? env <environment>))
@@ -189,6 +169,7 @@
       (compile-primitive-object compiler env repr '())))
 
 
+;; TBD: Remove the following procedure.
 (define (parse-type-expr compiler env type-expr)
   (assert (hrecord-is-instance? compiler <compiler>))
   (assert (hrecord-is-instance? env <environment>))
@@ -242,8 +223,18 @@
 	  (let ((r-super
 		 (tno-field-ref val 'cl-superclass))
 		(name (tno-field-ref val 'str-name)))
+
+	    ;; TBR
+	    (dwl1 "compute-class-decl/1")
+	    (dwl1 name)
+	    
 	    (let ((p-super
 		   (theme-compile-repr compiler env r-super)))
+
+	      ;; TBR
+	      (dwl1 "compute-class-decl/2")
+	      (dwl1 name)
+	    
 	      (let ((result
 		     `(define-class
 			,name
@@ -256,6 +247,11 @@
 			,(tno-field-ref val 'immutable?)
 			,(tno-field-ref val 'eq-by-value?)
 			,(tno-field-ref val 's-ctr-access))))
+
+		;; TBR
+		(dwl1 "compute-class-decl/3")
+		(dwl1 name)
+	    
 		result))))
 	 (else (raise 'internal-error-in-class))))))
 
@@ -333,41 +329,10 @@
   (assert (is-entity? value-expr))
   (dwl2 "compute-variable-def/0")
   (dwl2 (hfield-ref (hfield-ref var 'address) 'source-name))
-
-  ;; TBR
-  ;; (if gl-flag10?
-  ;;     (begin
-  ;; 	(dvar1-set! env)
-  ;; 	(dvar2-set! var)
-  ;; 	(dvar3-set! value-expr)
-  ;; 	(raise 'stop10)))
-
-  ;; It is not yet clear which expressions
-  ;; should be read when importing module interfaces.
-  ;; Probably the following three expression types
-  ;; should not occur in module interfaces.
-
-;;  (if gl-flag10? (dp))
-
   (let* ((interface-read-expr?
 	  ;; Not sure if incomplete objects work here.
 	  (is-target-object? value-expr))
-	  ;; (and (is-static-entity? value-expr)
-	  ;;      (not (hrecord-is-instance? value-expr <procedure-expression>))
-	  ;;      (not (hrecord-is-instance? value-expr <prim-proc-ref>))
-	  ;;      (not (hrecord-is-instance? value-expr <checked-prim-proc>))))
-
-	 ;; TBR
-;;	 (tmp1 (begin (dwl2 "compute-variable-def/1")
-;;		      (if gl-flag10? (dp)) 0))
-
 	 (t-type-decl (theme-compile-repr compiler env type-decl))
-
-	 ;; TBR
-;;	 (tmp2 (begin (dwl2 "compute-variable-def/2")
-;;		      (dwl2 (hrecord-type-name-of value-expr))
-;;		      (if gl-flag10? (dp)) 0))
-
 	 (t-value-expr (theme-compile-repr compiler env value-expr)))
     (dwl2 "compute-variable-def/3")
 ;;    (if gl-flag10? (dp))
@@ -415,13 +380,10 @@
   (strong-assert (hrecord-is-instance? expr <variable-definition>))
 
   ;; TBR
-  ;; (if (eq? (hfield-ref (hfield-ref (hfield-ref expr 'variable) 'address)
-  ;; 		       'source-name)
-  ;; 	   '<nonempty-list>)
-  ;;     (begin
-  ;; 	(dvar1-set! expr)
-  ;; 	(raise 'stop100)))
-
+  (dwl1 "compute-general-variable-def")
+  (dwl1 (hfield-ref (hfield-ref (hfield-ref expr 'variable) 'address)
+		    'source-name))
+    
   (let* ((var (hfield-ref expr 'variable))
 	 (r-address (hfield-ref var 'address))
 	 (p-address (theme-compile-address r-address))
@@ -432,27 +394,7 @@
 	 (read-only? (hfield-ref var 'read-only?))
 	 (volatile? (hfield-ref var 'volatile?))
 	 (contents
-	    (do-compute-general-variable-def compiler env expr)))
-	  ;; (cond
-	  ;;  ((not-null? (hfield-ref var 'value))
-	  ;;   (dwl4 "compute-general-variable-def/1")
-	  ;;   (let* ((value (hfield-ref var 'value))
-	  ;; 	   (exported-objects (hfield-ref compiler 'exported-objects))
-	  ;; 	   (prev (assv value exported-objects)))
-	  ;;     (if (not (eqv? prev #f))
-	  ;; 	  (let* ((prev-var (cdr prev))
-	  ;; 		 (prev-address (hfield-ref prev-var 'address)))
-	  ;; 	    (list 'obj-ref (theme-compile-address prev-address)))
-	  ;; 	  (let ((result (do-compute-general-variable-def
-	  ;; 			 compiler env expr)))
-	  ;; 	    (hfield-set! compiler 'exported-objects
-	  ;; 			 (cons (cons value var)
-	  ;; 			       (hfield-ref compiler
-	  ;; 					   'exported-objects)))
-	  ;; 	    result))))
-	  ;;  (else
-	  ;;   (do-compute-general-variable-def compiler env expr)))))
-    (dwl2 "compute-general-variable-def/2")    
+	  (do-compute-general-variable-def compiler env expr)))
     (list name forward-decl? declared?
 	  p-address exact-type? read-only? volatile? contents)))
 
@@ -624,19 +566,6 @@
 	    ,p-static-arg-types
 	    ,(hfield-ref repr 'runtime-arglist-typecheck?)
 	    ,p-default-params)))
-
-    ;; TBR
-    ;; (let ((address (hfield-ref (hfield-ref repr 'proc) 'address)))
-    ;;   (if (and (not-null? address)
-    ;; 	       (eq? (hfield-ref address 'source-name)
-    ;; 		    'cons))
-    ;; 	  (begin
-    ;; 	    (dvar1-set! repr)
-    ;; 	    (dvar2-set! env)
-    ;; 	    (dvar3-set! p-static-arg-types)
-    ;; 	    (dvar4-set! result)
-    ;; 	    (raise 'stop-appl))))
-
     (dwl3 "cr-proc-appl EXIT")
     result))
 
@@ -729,7 +658,8 @@
     (assert (not-null? (hfield-ref procexpr 'address)))
     (let* ((t-gen-proc (theme-compile-address (hfield-ref gen-proc 'address)))
 	   (m-address (hfield-ref procexpr 'address))
-;;	   (t-m-address (theme-compile-address m-address))
+	   ;;	   (t-m-address (theme-compile-address m-address))
+	   ;; TBD: Remove the following variable.
 	   (name (hfield-ref m-address 'source-name))
 	   (type (get-entity-type procexpr))
 	   (t-type (theme-compile-repr compiler env type))
@@ -744,13 +674,6 @@
   (assert (hrecord-is-instance? compiler <compiler>))
   (assert (hrecord-is-instance? env <environment>))
   (assert (hrecord-is-instance? repr <method-declaration>))
-
-  ;; TO BE REMOVED
-  ;; (set! gl-ctr4 (+ gl-ctr4 1))
-  ;; (dwl4 gl-ctr4)
-  ;; (dvar1-set! repr)
-  ;; (raise 'stop+)
-
   (let ((gen-proc (hfield-ref repr 'gen-proc))
 	(method (hfield-ref repr 'method)))
     (assert (not-null? (hfield-ref gen-proc 'address)))
@@ -1038,18 +961,6 @@
   (assert (hrecord-is-instance? env <environment>))
   (assert (hrecord-is-instance? repr <forward-declaration>))
   (dvar1-set! repr)
-
-  ;; TO BE REMOVED
-  ;; (dwl3 (hfield-ref (hfield-ref (hfield-ref repr 'variable) 'address)
-  ;; 		    'source-name))
-  ;; (if (eq? (hfield-ref (hfield-ref (hfield-ref repr 'variable) 'address)
-  ;; 		    'source-name)
-  ;; 	   'assoc)
-  ;;     (begin
-  ;; 	(dvar1-set! env)
-  ;; 	(dvar2-set! repr)
-  ;; 	(raise 'assoc-stop)))
-
   (let* ((variable (hfield-ref repr 'variable))
 	 (read-only? (hfield-ref variable 'read-only?))
 	 (volatile? (hfield-ref variable 'volatile?))
@@ -1059,17 +970,9 @@
 	   ((and read-only? volatile?) (raise 'internal-error))
 	   (volatile? 'declare-volatile)
 	   (else 'declare-mutable)))
-
-	 ;; TBR
-	 (tmp1 (begin (dwl3 "cr-forward-declaration/1") 0))
-
 	 (p-location (theme-compile-declared-variable
 		      compiler env variable))
 	 (r-declared-type (hfield-ref repr 'declared-type))
-
-	 ;; TBR
-	 (tmp2 (begin (dwl3 "cr-forward-declaration/2") 0))
-
 	 (p-declared-type (theme-compile-repr compiler env r-declared-type)))
     (dwl3 "cr-forward-declaration EXIT")
     `(,s-keyword ,p-location ,p-declared-type)))
@@ -1336,27 +1239,6 @@
     (list 'tvar (theme-compile-address address))))
 
 
-;; (define (co-primitive-object compiler env to lst-visited)
-;;   (assert (hrecord-is-instance? compiler <compiler>))
-;;   (assert (hrecord-is-instance? env <environment>))
-;;   (assert (is-target-object? to))
-;;   (assert (hfield-ref to 'primitive?))
-;;   (let ((p-type (theme-compile-object-fwd compiler env (get-entity-type to)
-;; 					  lst-visited)))
-;;     (cond
-;;      ((eqv? to to-nil)
-;;       (list 'primitive-atom p-type '()))
-;;      ((is-t-atomic-object? to)
-;;       (list 'primitive-atom p-type (hfield-ref to 'obj-prim-contents)))
-;;      (else
-;;       (assert (eqv? (get-entity-type (get-entity-type to))
-;; 		    tpc-pair))
-;;       (list 'primitive-value
-;; 	    p-type
-;; 	    (compile-quoted-object to)
-;; 	    (hfield-ref to 'l-opt-contents))))))
-
-
 (define (co-incomplete-object compiler env to lst-visited)
   (assert (hrecord-is-instance? compiler <compiler>))
   (assert (hrecord-is-instance? env <environment>))
@@ -1412,15 +1294,6 @@
   (assert (hrecord-is-instance? compiler <compiler>))
   (assert (hrecord-is-instance? env <environment>))
   (assert (is-t-param-class-instance? to))
-
-  ;; TBR
-  ;; (set! gl-counter20 (+ gl-counter20 1))
-  ;; (dwl3 gl-counter20)
-  ;; (dvar1-set! env)
-  ;; (dvar2-set! to)
-  ;; (dvar3-set! lst-visited)
-  ;; (raise 'stop-pci)
-
   (let* ((param-class (get-entity-type to))
 	 (args (tno-field-ref to 'l-tvar-values))
 	 (p-param-class (theme-compile-object-fwd compiler env param-class
@@ -1479,6 +1352,8 @@
 	      (list p-args))))
 
 
+;; This procedure is probably erroneous. Procedure co-param-class-instance
+;; is used instead.
 (define (co-vector-class compiler env to lst-visited)
   (assert (hrecord-is-instance? compiler <compiler>))
   (assert (hrecord-is-instance? env <environment>))
@@ -1490,6 +1365,8 @@
     result))
 
 
+;; This procedure is probably erroneous. Procedure co-param-class-instance
+;; is used instead.
 (define (co-mutable-vector-class compiler env repr lst-visited)
   (assert (hrecord-is-instance? compiler <compiler>))
   (assert (hrecord-is-instance? env <environment>))
@@ -1501,6 +1378,8 @@
     result))
 
 
+;; This procedure is probably erroneous. Procedure co-param-class-instance
+;; is used instead.
 (define (co-value-vector-class compiler env repr lst-visited)
   (assert (hrecord-is-instance? compiler <compiler>))
   (assert (hrecord-is-instance? env <environment>))
@@ -1512,6 +1391,8 @@
     result))
 
 
+;; This procedure is probably erroneous. Procedure co-param-class-instance
+;; is used instead.
 (define (co-mutable-value-vector-class compiler env repr lst-visited)
   (assert (hrecord-is-instance? compiler <compiler>))
   (assert (hrecord-is-instance? env <environment>))
@@ -1649,7 +1530,7 @@
 
 
 (define (compile-cycle compiler env to lst-visited)
-  (dwli2 "compile-cycle ENTER")
+  (dwl1 "compile-cycle ENTER")
   (assert (hrecord-is-instance? compiler <compiler>))
   (assert (hrecord-is-instance? env <environment>))
   (assert (is-target-object? to))
@@ -1661,23 +1542,34 @@
     (hfield-set! compiler 'lst-enclosing-cycles
 		 (cons (cons to address)
 		       (hfield-ref compiler 'lst-enclosing-cycles)))
+    (dwl1 "compile-cycle HEP")
     (let ((p-contents (theme-compile-object0-fwd compiler env to lst-visited
 						 #t)))
       (hfield-set! compiler 'lst-enclosing-cycles lst-old-encl)
-      (dwli2 "compile-cycle EXIT")
+      (dwl1 "compile-cycle EXIT")
       (list 'cycle p-type p-address p-contents))))
     
 
 (define (do-compile-object compiler env to lst-visited compile-always?)
   (dwli2 "do-compile-object")
+
+  ;; TBR
+  (dwl1 "do-compile-object")
+  (dwl1 compile-always?)
+  (if (is-target-object? to)
+      (let ((address (hfield-ref to 'address)))
+	(if (not-null? address)
+	    (dwl1 (hfield-ref address 'source-name)))))
+  
   (let ((result
 	 (cond
+	  ;; It may be unnecessary to check if to is null.
 	  ((null? to) '())
 	  ((is-t-type-variable? to)
 	   (co-type-variable compiler env to))
 	  ((and (not compile-always?)
 		(not-null? (hfield-ref to 'address)))
-	   	     (co-object-with-address compiler env to lst-visited))
+	   (co-object-with-address compiler env to lst-visited))
 	  ((hfield-ref to 'primitive?)
 	   (compile-primitive-object compiler env to lst-visited))
 	  ;; The following may be an error situation.
@@ -1743,19 +1635,21 @@
   (assert (hrecord-is-instance? env <environment>))
   (assert (is-target-object? to))
 
+  ;; TBR
+  (dwl1 "theme-compile-object0")
+  (dwl1 compile-always?)
+  (let ((address (hfield-ref to 'address)))
+    (if (not-null? address)
+	(dwl1 (hfield-ref address 'source-name))))
+  
   (let ((old-indent gl-indent))
     (set! gl-indent (+ gl-indent 1))
-
-    ;; TO BE REMOVED
-    ;; (ptno2 to)
-    ;;    (dvar1-set! to)
-    ;;    (dwli2 (target-object-as-string to '()))
-
     (let ((lst-new-visited (cons to lst-visited)))
 
       (let ((result
 	     (if compile-always?
 		 (do-compile-object compiler env to lst-new-visited #t)
+		 ;; TBD: Change assv to assq (?).
 		 (let* ((a (assv to
 				 (hfield-ref compiler 'lst-enclosing-cycles))))
 		   (if a
@@ -1781,54 +1675,20 @@
 
 (define (theme-compile-repr compiler env repr)
   (dwl4 "theme-compile-repr ENTER")
-  (dwl4 (hrecord? repr))
-  (dwl4 (hrecord-type-get-name (hrecord-type-of repr)))
   (assert (hrecord-is-instance? env <environment>))
-  (dwli2 "theme-compile-repr ENTER")
-  (dwli2 (hrecord-type-get-name (hrecord-type-of repr)))
-
-  ;; TBR
-  (if (and (hrecord-is-instance? repr <variable-definition>)
-  	   (eqv? (hfield-ref
-  		  (hfield-ref
-  		   (hfield-ref repr 'variable)
-  		   'address)
-  		  'source-name)
-  		 'split-string))
-      (dwli2 "split HEP"))
-
   (let ((old-indent gl-indent)
 	(prim? (is-t-primitive-object? repr)))
     (set! gl-indent (+ gl-indent 1))
-
-    ;; TBR
-    ;; (if (= gl-ctr4 10)
-    ;; 	(begin
-    ;; 	  (set! gl-ctr15 (+ gl-ctr15 1))
-    ;; 	  (display gl-ctr15)
-    ;; 	  (newline)))
-;;	  (display "Continue? ")
-;;	  (read)))
-    ;; (if (= gl-ctr15 15)
-    ;;     (begin
-    ;;       (dvar1-set! repr)
-    ;;       (dvar2-set! env)
-    ;;       (raise 'stop15)))
-    ;; (if (= gl-ctr15 39)
-    ;;     (begin
-    ;;       (display "Continue? ")
-    ;;       (read)))))
-    ;; (if (= gl-ctr15 40)
-    ;; 	(begin
-    ;; 	  (dvar1-set! repr)
-    ;; 	  (raise 'stop39)))
-
     (assert
      (or
       prim?
       (hrecord-is-instance? repr <entity>)))
+
+    ;; TBR
+    (dwl1 "theme-compile-repr")
     (if (hrecord? repr)
-	(dwli2 (hrecord-type-name-of repr)))
+	(dwl1 (hrecord-type-name-of repr)))
+
     (let ((prev-repr (hfield-ref compiler 'current-repr)))
       (if (is-entity? repr)
 	  (hfield-set! compiler 'current-repr repr))
@@ -1845,6 +1705,7 @@
 		       (dvar1-set! repr)
 		       (raise 'compilation-not-implemented)))))
 	      ((is-target-object? repr)
+	       (dwl1 "theme-compile-repr/1")
 	       (theme-compile-object compiler env repr '()))
 	      (else (raise 'illegal-element-to-compile)))))
 	(if (is-entity? repr)
@@ -1865,21 +1726,10 @@
   (dwli (length expr-list))
   (assert (hrecord-is-instance? env <environment>))
   (for-each (lambda (repr)
-
-	      ;; TBR
-	      (dwli "do-compile-unit/0-0")
-	      (set! gl-ctr4 (+ gl-ctr4 1))
-	      (dwli gl-ctr4)
-;;	      (if (= gl-ctr4 4) (set! gl-flag10? #t))
-
-	      ;; (debug-prompt-and-pause)
-	      ;; (if (= gl-ctr4 13)
-	      ;; 	  (begin
-	      ;; 	    (set! gl-show-indented-debug-info? #f)))
-	      ;; (display-time)
-
 	      (dwl3 "do-compile-unit/0")
 	      (hfield-set! compiler 'current-toplevel-repr repr)
+
+	      ;; TBR
 	      (dvar1-set! repr)
 	      (dwli (hrecord-type-get-name (hrecord-type-of repr)))
 	      (if (hrecord-is-instance? repr <variable-definition>)
@@ -1887,6 +1737,7 @@
 						'address)
 				    'source-name)))
 	      (dwli "do-compile-unit/1")
+	      
 	      ;; Not sure if we should reset ht-cycles for each expression.
 	      ;; (let* ((binder (compiler-get-binder compiler))
 	      ;; 	     (repr1 (handle-cycles binder repr))
@@ -1897,31 +1748,10 @@
 		(hfield-set! compiler 'lst-enclosing-cycles '())
 		(hfield-set! compiler 'lst-visited '())
 		(dwli "do-compile-unit/1-1")
-		(display-time)
+		;; (display-time)
 		(detect-cycles binder repr ht-cycles '())
 		(dwli "do-compile-unit/1-2")
-		(display-time)
-
-		;; TBR
-		;; (if (= gl-ctr4 3)
-		;;     (begin
-		;;       (dvar1-set! env)
-		;;       (dvar2-set! repr)
-		;;       (raise 'stop4)))
-
-		;; TO BE REMOVED
-		;; (if (and
-		;;      (hrecord-is-instance? repr <variable-definition>)
-		;;      (eq? (hfield-ref (hfield-ref (hfield-ref repr 'variable)
-		;; 				  'address)
-		;; 		      'source-name)
-		;; 	  'split-string))
-		;;     (begin
-		;;       (dvar1-set! repr)
-		;;       (dvar2-set! env)
-		;;       (dwli "split HEP")
-		;;       (raise 'u2-stop)))
-
+		;; (display-time)
 		(let ((p-repr (theme-compile-repr compiler env repr)))
 		  (guard
 		   (exc (else
@@ -1935,15 +1765,6 @@
 		       (write p-repr module-output-file))
 		   (dwli "do-compile-unit/3")
 		   (newline module-output-file)))
-		;;	      (p (theme-compile-repr compiler env repr)
-		;;		 body-output-file)
-		;;	      (display (theme-compile-repr env repr) body-output-file)
-		;;	      (newline body-output-file)
-
-		;; (display-time)
-		;; (if (= gl-ctr4 13)
-		;;     (raise 'stop4))
-
 		(hfield-set! compiler 'current-toplevel-repr '())))
 	    expr-list)
   (dwl3 "do-compile-unit EXIT"))
