@@ -1946,14 +1946,6 @@
   (is-t-param-signature? (get-entity-type ent)))
 
 
-(define (entity-is-none? ent)
-  (or
-   ;; The following test is an optimization.
-   (eq? ent tt-none)
-   ;; The only type that inherits from <none> is <none>.
-   (is-t-subtype? gl-binder ent tt-none)))
-
-
 (define (entity-is-none1? binder ent)
   (or
    ;; The following test is an optimization.
@@ -1962,31 +1954,31 @@
    (is-t-subtype? binder ent tt-none)))
 
 
-(define (entity-type-is-none? ent)
-  (entity-is-none? (get-entity-type ent)))
-
-
 (define (entity-type-is-none1? binder ent)
   (entity-is-none1? binder (get-entity-type ent)))
 
 
-(define (check-no-none-arguments arguments s-proc-name)
+(define (contains-none-value? binder l)
+  (or-map? (lambda (ent) (entity-type-is-none1? binder ent)) l))
+
+
+(define (check-no-none-arguments binder arguments s-proc-name)
   (if (and (not-null? arguments)
-	   (or-map? entity-type-is-none? arguments))
+	   (contains-none-value? binder arguments))
       (raise (list 'procedure-argument-with-type-none
 		   (cons 's-proc-name s-proc-name)))))
 
 
-(define (check-no-none-types? types)
+(define (check-no-none-types? binder types)
   (and-map?
-   (lambda (type) (not (entity-is-none? type)))
+   (lambda (type) (not (entity-is-none1? binder type)))
    types))
 
 
-(define (check-no-none-types types)
-  (if (and (not-null? types)
-	   (or-map? entity-is-none? types))
-      (raise 'illegal-use-of-none)))
+;; (define (check-no-none-types types)
+;;   (if (and (not-null? types)
+;; 	   (or-map? entity-is-none? types))
+;;       (raise 'illegal-use-of-none)))
 
 
 (define (make-unknown-object type exact-type?)
