@@ -25,16 +25,6 @@
 ;;  (string-append "-u-" (symbol->string module-name) "-"))
 
 
-;; We use param-cache-parsing here because the target compilation
-;; uses data structures created in implementation pseudocode reading.
-;; Data structures created during instantiation are discarded.
-(define (get-binder-for-tc linker)
-  (hfield-ref linker 'binder-parsing))
-
-
-(set! get-binder-for-tc-fwd get-binder-for-tc)
-
-
 (define (maybe-symbol->string sym)
   (if (not-null? sym) (symbol->string sym) ""))
 
@@ -94,6 +84,20 @@
   `(check-letrec-unspecified ,expr (quote ,s-var-name)))
 
 
+(define (tc-scheme-call-stack-debug linker repr t-body)
+  (if (hfield-ref linker 'runtime-pretty-backtrace?)
+      (let* ((s-kind (hfield-ref repr 's-kind))
+	     (s-name (hfield-ref repr 's-name))
+	     (l-module (hfield-ref repr 'l-module)))
+	`(begin
+	   (call-stack-push (quote ,s-kind) (quote ,s-name)
+			    (quote ,l-module))
+	   (let ((result ,t-body))
+	     (call-stack-pop)
+	     result)))
+      t-body))
+
+
 (define (tcomp-primitive-object linker repr)
   (dwli2 "tcomp-primitive-object")
   (assert (hfield-ref repr 'primitive?))
@@ -213,7 +217,7 @@
   (let ((name (get-target-var-name linker address))
 	(t-result-type (theme-target-compile linker result-type))
 	(source-name (hfield-ref address 'source-name)))
-    (if (entity-is-none? result-type)
+    (if (linker-entity-is-none? linker result-type)
 	`(lambda (,@t-tvars . arguments)
 	   (apply ,name arguments))
 	`(lambda (,@t-tvars . arguments)
@@ -333,7 +337,11 @@
 	 (tmp1 (begin (dwli "compile-proc-expr/1") 0))
 	 (body (hfield-ref repr 'body))
 	 (tmp2 (begin (dwli "compile-proc-expr/2") 0))
-	 (c-body (theme-target-compile linker body))
+	 (c-body
+	  (tc-scheme-call-stack-debug
+	   linker
+	   repr
+	   (theme-target-compile linker body)))
 	 (tmp3 (begin (dwli "compile-proc-expr/3") 0))
 	 (type (get-entity-type repr))
 	 (tmp4 (begin (dwli "compile-proc-expr/4") 0))
@@ -346,7 +354,7 @@
 	(list '_i_make-procedure c-type
 	      (construct-procedure c-body args c-arg-descs c-result-type
 				   simple?))
-	(let ((no-result? (entity-is-none? result-type))
+	(let ((no-result? (linker-entity-is-none? linker result-type))
 	      (type-dispatched? (entity-type-dispatched? body)))
 	  (construct-param-proc-body linker
 				     t-tvars
@@ -407,7 +415,7 @@
 	     ;; We must not check the result value
 	     ;; if the result type is <none>.
 	     (let ((result-type (tno-field-ref type 'type-result)))
-	       (if (entity-is-none? result-type)
+	       (if (linker-entity-is-none? linker result-type)
 		   (list '_i_make-procedure
 			 (tcomp-object-fwd linker (get-entity-type repr) '() #f)
 			 (get-target-var-name linker (hfield-ref repr 'address)))
@@ -423,7 +431,7 @@
 				  'type-result))
 		    (tvars (tno-field-ref type 'l-tvars))
 		    (t-tvars (compile-type-vars linker tvars)))
-	       (if (entity-is-none? result-type)
+	       (if (linker-entity-is-none? linker result-type)
 		   (let ((name (get-target-var-name linker
 						    (hfield-ref repr 'address))))
 		     (list '_i_make-param-proc
@@ -1112,7 +1120,7 @@
 			     (theme-target-compile linker repr-arg))
 			   (hfield-ref repr 'arglist))))))
     (if (and (entity-type-dispatched? repr)
-	     (not (entity-is-none? (get-entity-type repr))))
+	     (not (linker-entity-is-none? linker (get-entity-type repr))))
 	(let ((t-type-expr (theme-target-compile
 			    linker (get-entity-type repr))))
 	  (generate-type-check linker t-expr t-type-expr))
@@ -1206,7 +1214,9 @@
 			      (cons 'list t-args)
 			      t-static)))
 		  (if (and (entity-type-dispatched? repr)
-			   (not (entity-is-none? (get-entity-type repr))))
+			   (not (linker-entity-is-none?
+				 linker
+				 (get-entity-type repr))))
 		      (let ((t-type-expr (theme-target-compile
 					  linker (get-entity-type repr))))
 			(generate-type-check linker t-expr t-type-expr))
@@ -1799,7 +1809,8 @@
 	 (appl-pure? (hfield-ref repr 'appl-pure?))
 	 (appl-always-returns? (hfield-ref repr 'appl-always-returns?))
 	 (appl-never-returns? (hfield-ref repr 'appl-never-returns?)))
-    (if (and (entity-type-dispatched? repr) (not (entity-is-none? result-type)))
+    (if (and (entity-type-dispatched? repr)
+	     (not (linker-entity-is-none? linker result-type)))
 	(let ((t-result-type (theme-target-compile linker result-type)))
 	  (list '_i_dispatch-generic-proc
 		(theme-target-compile linker (hfield-ref repr 'generic-proc))
@@ -1914,6 +1925,16 @@
       '(quote ())))
 
 
+(define (tcomp-debug-output linker repr)
+  (dwli2 "tcomp-debug-output")
+  (assert (hrecord-is-instance? linker <linker>))
+  (assert (hrecord-is-instance? repr <debug-output-expr>))
+  (let ((x-message (hfield-ref repr 'x-message)))
+    (if (or (null? x-message) (symbol? x-message) (pair? x-message))
+	`(theme-debug-print (quote ,x-message))
+	`(theme-debug-print ,x-message))))
+
+
 (define (tcomp-empty linker repr)
   (assert (hrecord-is-instance? linker <linker>))
   (assert (hrecord-is-instance? repr <empty-expression>))
@@ -1968,6 +1989,7 @@
    (cons <zero-setting-expr> tcomp-zero-setting)
    (cons <force-pure-expr> tcomp-force-pure-expr)
    (cons <assertion-expr> tcomp-assertion)
+   (cons <debug-output-expr> tcomp-debug-output)
    (cons <empty-expression> tcomp-empty)
    (cons <normal-variable> tcomp-error)
    (cons <target-object> tcomp-error)))
@@ -2452,7 +2474,7 @@
 		  '()
 		  (let ((param-class (list-ref instance 2))
 			(params (list-ref instance 3))
-			(binder (get-binder-for-tc-fwd linker))
+			(binder (get-binder-for-tc linker))
 			(ht-cycles (hfield-ref linker 'ht-cycles)))
 		      (hash-clear! ht-cycles)
 ;;		      (detect-cycles binder to ht-cycles '())
@@ -2476,7 +2498,7 @@
 	     ((proc)
 	      (assert (= (length instance) 3))
 	      (let ((expr (list-ref instance 2))
-		    (binder (get-binder-for-tc-fwd linker))
+		    (binder (get-binder-for-tc linker))
 		    (ht-cycles (hfield-ref linker 'ht-cycles)))
 
 		;; TBR
@@ -2545,7 +2567,7 @@
 		      linker
 		      (hfield-ref fact-expr 'address)))
 	   (r-value-expr (hfield-ref fact-expr 'to))
-	   (binder (get-binder-for-tc-fwd linker))
+	   (binder (get-binder-for-tc linker))
 	   (ht-cycles (hfield-ref linker 'ht-cycles)))
       (hash-clear! ht-cycles)
       (detect-cycles binder r-value-expr ht-cycles '())
@@ -2750,7 +2772,7 @@
 ;;	   (hfield-set! linker 'current-toplevel-repr entity)
 	   (dwl2 "tc-scheme-entity/2")
 	   (dwl2 (hrecord-type-name-of entity))
-	   (let ((binder (get-binder-for-tc-fwd linker))
+	   (let ((binder (get-binder-for-tc linker))
 		 (ht-cycles (hfield-ref linker 'ht-cycles)))
 	     (hash-clear! ht-cycles)
 	     (detect-cycles binder entity ht-cycles '())
@@ -2786,7 +2808,7 @@
   (let* ((my-main (get-main linker))
 	 (result-type
 	  (tno-field-ref (get-entity-type my-main) 'type-result)))
-    (if (entity-is-none? result-type)
+    (if (linker-entity-is-none? linker result-type)
 	;; We could change result value '() to some platform dependent
 	;; object meaning undefined value.
 	'(define main
