@@ -414,7 +414,7 @@
   (assert (is-binder? binder))
   (assert (is-entity? obj-expr))
   (strong-assert (symbol? field-name))
-  (strong-assert (not (entity-type-is-none? obj-expr)))
+  (strong-assert (not (entity-type-is-none1? binder obj-expr)))
   (strong-assert (is-t-instance? binder (get-entity-type obj-expr) tc-class))
   (let* ((type (get-entity-type obj-expr))
 	 (field (get-field-spec type field-name)))
@@ -456,8 +456,8 @@
   (strong-assert (symbol? field-name))
   (assert (is-entity? value))
   (assert (boolean? type-check?))
-  (strong-assert (and (not (entity-type-is-none? obj-expr))
-		      (not (entity-type-is-none? value))))
+  (strong-assert (and (not (entity-type-is-none1? binder obj-expr))
+		      (not (entity-type-is-none1? binder value))))
   (dwl4 "translate-const-field-set/1")
   (let* ((type-expr (get-entity-type obj-expr))
 	 (type-dispatched? (entity-type-dispatched? obj-expr))
@@ -536,7 +536,7 @@
   (assert (is-entity? proc))
   (assert (and (list? arguments)
 	       (and-map? is-entity? arguments)))
-  (check-no-none-arguments arguments 'tuple-type-with-tail)
+  (check-no-none-arguments binder arguments 'tuple-type-with-tail)
   (if (= (length arguments) 2)
       ;; What about purity?
       (let ((tuple-part (car arguments))
@@ -556,7 +556,7 @@
   (assert (is-entity? proc))
   (assert (and (list? arguments)
 	       (and-map? is-entity? arguments)))
-  (check-no-none-arguments arguments 'class-of)
+  (check-no-none-arguments binder arguments 'class-of)
   (if (= (length arguments) 1)
       (let* ((r-value (car arguments))
 	     (r-type (get-entity-type r-value)))
@@ -601,7 +601,7 @@
   (assert (and (list? arguments)
 	       (and-map? is-entity? arguments)))
   (assert (boolean? inside-param-def?))
-  (check-no-none-arguments arguments 'is-subtype?)
+  (check-no-none-arguments binder arguments 'is-subtype?)
   (dwl1 "translate-is-subtype-appl")
   (if (= (length arguments) 2)
       (let* ((r-type1 (car arguments))
@@ -615,6 +615,7 @@
 	  (raise 'illegal-free-type-variables))
 	 ((and (not free-tvars?) 
 	       (and-map? is-known-pure-entity? arguments))
+	  ;; TBD: use get-entity-value in the following statement.
 	  (if (and (is-type2? binder r-type1)
 		   (is-type2? binder r-type2))
 	      (make-primitive-object
@@ -657,12 +658,13 @@
   (assert (is-entity? proc))
   (assert (and (list? arguments)
 	       (and-map? is-entity? arguments)))
-  (check-no-none-arguments arguments 'is-instance?)
+  (check-no-none-arguments binder arguments 'is-instance?)
   (if (= (length arguments) 2)
       (let* ((r-object (car arguments))
 	     (r-type (cadr arguments))
 	     (r-object-type (get-entity-type r-object))
 	     (type-dispatched? (entity-type-dispatched? r-object))
+	     ;; TBD: should we change this: 
 	     ;; (contains-free-tvars?
 	     ;;  (or (contains-free-tvars-general-fwd? r-object-type)
 	     ;; 	  (contains-free-tvars-general-fwd? r-type))))
@@ -677,8 +679,9 @@
 	   (and-map? is-known-pure-entity? arguments)
 	   type-dispatched?
 	   (not free-tvars?))
-	  ;; The following check should never fail.
+	  ;; The following check should never fail.	
 	  (assert (is-type2? binder r-object-type))
+	  ;; TBD: use the value of r-type in the following.
 	  (if (is-type2? binder r-type)
 	      (make-primitive-object
 	       tc-boolean
@@ -695,6 +698,7 @@
 			  #t
 			  #t
 			  '()
+			  ;; TBD: Change the following to pure-args?.
 			  #t
 			  #f
 			  (not correct-type?)
@@ -719,7 +723,7 @@
   (strong-assert (and (list? arguments)
 		      (and-map? is-entity? arguments)
 		      (= (length arguments) 2)))
-  (check-no-none-arguments arguments 'tuple-ref)
+  (check-no-none-arguments binder arguments 'tuple-ref)
   (let* ((r-tuple (car arguments))
 	 (r-type (get-entity-type r-tuple))
 	 (r-index (cadr arguments))
@@ -803,7 +807,7 @@
   (assert (boolean? eq-by-value?))
   (assert (boolean? type-check?))
   (assert (boolean? inside-param-def?))
-  (check-no-none-arguments arguments
+  (check-no-none-arguments binder arguments
 			   (if mutable?
 			       (if eq-by-value?
 				   'make-mutable-value-vector
@@ -898,7 +902,7 @@
   (assert (boolean? eq-by-value?))
   (assert (boolean? type-check?))
   (assert (boolean? inside-param-def?))
-  (check-no-none-arguments arguments
+  (check-no-none-arguments binder arguments
 			   (if mutable?
 			       (if eq-by-value?
 				   'mutable-value-vector
@@ -1071,7 +1075,7 @@
 	       (and-map? is-entity? arguments)))
   (assert (boolean? type-check?))
   (assert (boolean? inside-param-def?))
-  (check-no-none-arguments arguments
+  (check-no-none-arguments binder arguments
 			   (if mutable?
 			       (if value-vector?
 				   'cast-mutable-value-vector
@@ -1153,7 +1157,7 @@
 	       (and-map? is-entity? arguments)))
   (assert (boolean? type-check?))
   (assert (boolean? inside-param-def?))
-  (check-no-none-arguments arguments
+  (check-no-none-arguments binder arguments
 			   (if mutable?
 			       (if value-vector?
 				   'cast-mutable-value-vector-metaclass
