@@ -181,7 +181,7 @@
 						type-arg)
   (assert (is-binder? binder))
   (assert (is-entity? type-arg))
-  (if (entity-is-none? type-arg)
+  (if (entity-is-none1? binder type-arg)
       (raise 'none-as-uniform-list-member-type)
       (let* ((type-arg-list (construct-toplevel-type-repr
 			     binder
